@@ -7,6 +7,7 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
 
 # Set default logging handler to avoid "No handler found" warnings.
 import logging
@@ -21,6 +22,6 @@ class UrllibWarningFilter:
 
 logging.getLogger('urllib3.connectionpool').addFilter(UrllibWarningFilter())
 
-import b2sdk.version
+import b2sdk.version  # noqa: E402
 __version__ = b2sdk.version.VERSION
 assert __version__  # PEP-0396
