@@ -7,24 +7,31 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
+
 import logging
-from typing import Optional, Tuple, List, Generator
 from contextlib import suppress
+from typing import Generator
 
 from .account_info.abstract import AbstractAccountInfo
 from .account_info.exception import MissingAccountData
-from .api_config import B2HttpApiConfig, DEFAULT_HTTP_API_CONFIG
+from .api_config import DEFAULT_HTTP_API_CONFIG, B2HttpApiConfig
 from .application_key import ApplicationKey, BaseApplicationKey, FullApplicationKey
-from .cache import AbstractCache
 from .bucket import Bucket, BucketFactory
+from .cache import AbstractCache
 from .encryption.setting import EncryptionSetting
-from .replication.setting import ReplicationConfiguration
-from .exception import BucketIdNotFound, NonExistentBucket, RestrictedBucket, RestrictedBucketMissing
+from .exception import (
+    BucketIdNotFound,
+    NonExistentBucket,
+    RestrictedBucket,
+    RestrictedBucketMissing,
+)
 from .file_lock import FileRetentionSetting, LegalHold
 from .file_version import DownloadVersionFactory, FileIdAndName, FileVersion, FileVersionFactory
 from .large_file.services import LargeFileServices
-from .raw_api import API_VERSION
 from .progress import AbstractProgressListener
+from .raw_api import API_VERSION, LifecycleRule
+from .replication.setting import ReplicationConfiguration
 from .session import B2Session
 from .transfer import (
     CopyManager,
@@ -50,7 +57,7 @@ def url_for_api(info, api_name):
         base = info.get_download_url()
     else:
         base = info.get_api_url()
-    return '%s/b2api/%s/%s' % (base, API_VERSION, api_name)
+    return f'{base}/b2api/{API_VERSION}/{api_name}'
 
 
 class Services:
@@ -62,12 +69,12 @@ class Services:
     def __init__(
         self,
         api,
-        max_upload_workers: Optional[int] = None,
-        max_copy_workers: Optional[int] = None,
-        max_download_workers: Optional[int] = None,
-        save_to_buffer_size: Optional[int] = None,
+        max_upload_workers: int | None = None,
+        max_copy_workers: int | None = None,
+        max_download_workers: int | None = None,
+        save_to_buffer_size: int | None = None,
         check_download_hash: bool = True,
-        max_download_streams_per_file: Optional[int] = None,
+        max_download_streams_per_file: int | None = None,
     ):
         """
         Initialize Services object using given session.
@@ -128,15 +135,15 @@ class B2Api(metaclass=B2TraceMeta):
 
     def __init__(
         self,
-        account_info: Optional[AbstractAccountInfo] = None,
-        cache: Optional[AbstractCache] = None,
-        max_upload_workers: Optional[int] = None,
-        max_copy_workers: Optional[int] = None,
+        account_info: AbstractAccountInfo | None = None,
+        cache: AbstractCache | None = None,
+        max_upload_workers: int | None = None,
+        max_copy_workers: int | None = None,
         api_config: B2HttpApiConfig = DEFAULT_HTTP_API_CONFIG,
-        max_download_workers: Optional[int] = None,
-        save_to_buffer_size: Optional[int] = None,
+        max_download_workers: int | None = None,
+        save_to_buffer_size: int | None = None,
         check_download_hash: bool = True,
-        max_download_streams_per_file: Optional[int] = None,
+        max_download_streams_per_file: int | None = None,
     ):
         """
         Initialize the API using the given account info.
@@ -222,10 +229,10 @@ class B2Api(metaclass=B2TraceMeta):
         bucket_type,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
-        default_server_side_encryption: Optional[EncryptionSetting] = None,
-        is_file_lock_enabled: Optional[bool] = None,
-        replication: Optional[ReplicationConfiguration] = None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
+        default_server_side_encryption: EncryptionSetting | None = None,
+        is_file_lock_enabled: bool | None = None,
+        replication: ReplicationConfiguration | None = None,
     ):
         """
         Create a bucket.
@@ -234,7 +241,7 @@ class B2Api(metaclass=B2TraceMeta):
         :param str bucket_type: a bucket type, could be one of the following values: ``"allPublic"``, ``"allPrivate"``
         :param dict bucket_info: additional bucket info to store with the bucket
         :param dict cors_rules: bucket CORS rules to store with the bucket
-        :param list lifecycle_rules: bucket lifecycle rules to store with the bucket
+        :param lifecycle_rules: bucket lifecycle rules to store with the bucket
         :param b2sdk.v2.EncryptionSetting default_server_side_encryption: default server side encryption settings (``None`` if unknown)
         :param bool is_file_lock_enabled: boolean value specifies whether bucket is File Lock-enabled
         :param b2sdk.v2.ReplicationConfiguration replication: bucket replication rules or ``None``
@@ -255,21 +262,17 @@ class B2Api(metaclass=B2TraceMeta):
             replication=replication,
         )
         bucket = self.BUCKET_FACTORY_CLASS.from_api_bucket_dict(self, response)
-        assert name == bucket.name, 'API created a bucket with different name\
-                                     than requested: %s != %s' % (name, bucket.name)
-        assert bucket_type == bucket.type_, 'API created a bucket with different type\
-                                             than requested: %s != %s' % (
-            bucket_type, bucket.type_
-        )
+        assert name == bucket.name, f'API created a bucket with different name than requested: {name} != {name}'
+        assert bucket_type == bucket.type_, f'API created a bucket with different type than requested: {bucket_type} != {bucket.type_}'
         self.cache.save_bucket(bucket)
         return bucket
 
     def download_file_by_id(
         self,
         file_id: str,
-        progress_listener: Optional[AbstractProgressListener] = None,
-        range_: Optional[Tuple[int, int]] = None,
-        encryption: Optional[EncryptionSetting] = None,
+        progress_listener: AbstractProgressListener | None = None,
+        range_: tuple[int, int] | None = None,
+        encryption: EncryptionSetting | None = None,
     ) -> DownloadedFile:
         """
         Download a file with the given ID.
@@ -467,7 +470,7 @@ class B2Api(metaclass=B2TraceMeta):
         :param str file_id: a file ID
         """
         url = url_for_api(self.account_info, 'b2_download_file_by_id')
-        return '%s?fileId=%s' % (url, file_id)
+        return f'{url}?fileId={file_id}'
 
     def get_download_url_for_file_name(self, bucket_name, file_name):
         """
@@ -477,18 +480,18 @@ class B2Api(metaclass=B2TraceMeta):
         :param str file_name: a file name
         """
         self.check_bucket_name_restrictions(bucket_name)
-        return '%s/file/%s/%s' % (
+        return '{}/file/{}/{}'.format(
             self.account_info.get_download_url(), bucket_name, b2_url_encode(file_name)
         )
 
     # keys
     def create_key(
         self,
-        capabilities: List[str],
+        capabilities: list[str],
         key_name: str,
-        valid_duration_seconds: Optional[int] = None,
-        bucket_id: Optional[str] = None,
-        name_prefix: Optional[str] = None,
+        valid_duration_seconds: int | None = None,
+        bucket_id: str | None = None,
+        name_prefix: str | None = None,
     ):
         """
         Create a new :term:`application key`.
@@ -534,7 +537,7 @@ class B2Api(metaclass=B2TraceMeta):
         response = self.session.delete_key(application_key_id=application_key_id)
         return ApplicationKey.from_api_response(response)
 
-    def list_keys(self, start_application_key_id: Optional[str] = None
+    def list_keys(self, start_application_key_id: str | None = None
                  ) -> Generator[ApplicationKey, None, None]:
         """
         List application keys. Lazily perform requests to B2 cloud and return all keys.
@@ -557,7 +560,7 @@ class B2Api(metaclass=B2TraceMeta):
                 return
             start_application_key_id = next_application_key_id
 
-    def get_key(self, key_id: str) -> Optional[ApplicationKey]:
+    def get_key(self, key_id: str) -> ApplicationKey | None:
         """
         Gets information about a single key: it's capabilities, prefix, name etc
 
