@@ -7,13 +7,12 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
 
 import fnmatch
 import logging
 import pathlib
-
 from contextlib import suppress
-from typing import Dict, Optional, Tuple
 
 from .encryption.setting import EncryptionSetting, EncryptionSettingFactory
 from .encryption.types import EncryptionMode
@@ -34,13 +33,14 @@ from .file_lock import (
 )
 from .file_version import DownloadVersion, FileVersion
 from .progress import AbstractProgressListener, DoNothingProgressListener
+from .raw_api import LifecycleRule
 from .replication.setting import ReplicationConfiguration, ReplicationConfigurationFactory
 from .transfer.emerge.executor import AUTO_CONTENT_TYPE
-from .transfer.emerge.write_intent import WriteIntent
 from .transfer.emerge.unbound_write_intent import UnboundWriteIntentGenerator
+from .transfer.emerge.write_intent import WriteIntent
 from .transfer.inbound.downloaded_file import DownloadedFile
 from .transfer.outbound.copy_source import CopySource
-from .transfer.outbound.upload_source import UploadSourceBytes, UploadSourceLocalFile, UploadMode
+from .transfer.outbound.upload_source import UploadMode, UploadSourceBytes, UploadSourceLocalFile
 from .utils import (
     B2TraceMeta,
     Sha1HexDigest,
@@ -68,7 +68,7 @@ class Bucket(metaclass=B2TraceMeta):
         type_=None,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
         revision=None,
         bucket_dict=None,
         options_set=None,
@@ -76,8 +76,8 @@ class Bucket(metaclass=B2TraceMeta):
             EncryptionMode.UNKNOWN
         ),
         default_retention: BucketRetentionSetting = UNKNOWN_BUCKET_RETENTION,
-        is_file_lock_enabled: Optional[bool] = None,
-        replication: Optional[ReplicationConfiguration] = None,
+        is_file_lock_enabled: bool | None = None,
+        replication: ReplicationConfiguration | None = None,
     ):
         """
         :param b2sdk.v2.B2Api api: an API object
@@ -86,7 +86,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param str type_: a bucket type
         :param dict bucket_info: an info to store with a bucket
         :param dict cors_rules: CORS rules to store with a bucket
-        :param list lifecycle_rules: lifecycle rules of the bucket
+        :param lifecycle_rules: lifecycle rules of the bucket
         :param int revision: a bucket revision number
         :param dict bucket_dict: a dictionary which contains bucket parameters
         :param set options_set: set of bucket options strings
@@ -110,7 +110,7 @@ class Bucket(metaclass=B2TraceMeta):
         self.is_file_lock_enabled = is_file_lock_enabled
         self.replication = replication
 
-    def get_fresh_state(self) -> 'Bucket':
+    def get_fresh_state(self) -> Bucket:
         """
         Fetch all the information about this bucket and return a new bucket object.
         This method does NOT change the object it is called on.
@@ -128,7 +128,7 @@ class Bucket(metaclass=B2TraceMeta):
         """
         return self.id_
 
-    def set_info(self, new_bucket_info, if_revision_is=None) -> 'Bucket':
+    def set_info(self, new_bucket_info, if_revision_is=None) -> Bucket:
         """
         Update bucket info.
 
@@ -137,7 +137,7 @@ class Bucket(metaclass=B2TraceMeta):
         """
         return self.update(bucket_info=new_bucket_info, if_revision_is=if_revision_is)
 
-    def set_type(self, bucket_type) -> 'Bucket':
+    def set_type(self, bucket_type) -> Bucket:
         """
         Update bucket type.
 
@@ -147,16 +147,16 @@ class Bucket(metaclass=B2TraceMeta):
 
     def update(
         self,
-        bucket_type: Optional[str] = None,
-        bucket_info: Optional[dict] = None,
-        cors_rules: Optional[dict] = None,
-        lifecycle_rules: Optional[list] = None,
-        if_revision_is: Optional[int] = None,
-        default_server_side_encryption: Optional[EncryptionSetting] = None,
-        default_retention: Optional[BucketRetentionSetting] = None,
-        replication: Optional[ReplicationConfiguration] = None,
-        is_file_lock_enabled: Optional[bool] = None,
-    ) -> 'Bucket':
+        bucket_type: str | None = None,
+        bucket_info: dict | None = None,
+        cors_rules: dict | None = None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
+        if_revision_is: int | None = None,
+        default_server_side_encryption: EncryptionSetting | None = None,
+        default_retention: BucketRetentionSetting | None = None,
+        replication: ReplicationConfiguration | None = None,
+        is_file_lock_enabled: bool | None = None,
+    ) -> Bucket:
         """
         Update various bucket parameters.
 
@@ -199,9 +199,9 @@ class Bucket(metaclass=B2TraceMeta):
     def download_file_by_id(
         self,
         file_id: str,
-        progress_listener: Optional[AbstractProgressListener] = None,
-        range_: Optional[Tuple[int, int]] = None,
-        encryption: Optional[EncryptionSetting] = None,
+        progress_listener: AbstractProgressListener | None = None,
+        range_: tuple[int, int] | None = None,
+        encryption: EncryptionSetting | None = None,
     ) -> DownloadedFile:
         """
         Download a file by ID.
@@ -224,9 +224,9 @@ class Bucket(metaclass=B2TraceMeta):
     def download_file_by_name(
         self,
         file_name: str,
-        progress_listener: Optional[AbstractProgressListener] = None,
-        range_: Optional[Tuple[int, int]] = None,
-        encryption: Optional[EncryptionSetting] = None,
+        progress_listener: AbstractProgressListener | None = None,
+        range_: tuple[int, int] | None = None,
+        encryption: EncryptionSetting | None = None,
     ) -> DownloadedFile:
         """
         Download a file by name.
@@ -328,7 +328,7 @@ class Bucket(metaclass=B2TraceMeta):
         folder_to_list: str = '',
         latest_only: bool = True,
         recursive: bool = False,
-        fetch_count: Optional[int] = 10000,
+        fetch_count: int | None = 10000,
         with_wildcard: bool = False,
     ):
         """
@@ -488,13 +488,14 @@ class Bucket(metaclass=B2TraceMeta):
         data_bytes,
         file_name,
         content_type=None,
-        file_infos=None,
+        file_info: dict | None = None,
         progress_listener=None,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        large_file_sha1: Optional[Sha1HexDigest] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        large_file_sha1: Sha1HexDigest | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         """
         Upload bytes in memory to a B2 file.
@@ -505,13 +506,14 @@ class Bucket(metaclass=B2TraceMeta):
         :param bytes data_bytes: a byte array to upload
         :param str file_name: a file name to upload bytes to
         :param str,None content_type: the MIME type, or ``None`` to accept the default based on file extension of the B2 file name
-        :param dict,None file_infos: a file info to store with the file or ``None`` to not store anything
+        :param dict,None file_info: a file info to store with the file or ``None`` to not store anything
         :param b2sdk.v2.AbstractProgressListener,None progress_listener: a progress listener object to use, or ``None`` to not track progress
         :param b2sdk.v2.EncryptionSetting encryption: encryption settings (``None`` if unknown)
         :param b2sdk.v2.FileRetentionSetting file_retention: file retention setting
         :param bool legal_hold: legal hold setting
         :param Sha1HexDigest,None large_file_sha1: SHA-1 hash of the result file or ``None`` if unknown
         :param int,None custom_upload_timestamp: override object creation date, expressed as a number of milliseconds since epoch
+        :param str,None cache_control: an optional cache control setting. Syntax based on the section 14.9 of RFC 2616. Example string value: 'public, max-age=86400, s-maxage=3600, no-transform'.
         :rtype: b2sdk.v2.FileVersion
         """
         upload_source = UploadSourceBytes(data_bytes)
@@ -519,13 +521,14 @@ class Bucket(metaclass=B2TraceMeta):
             upload_source,
             file_name,
             content_type=content_type,
-            file_info=file_infos,
+            file_info=file_info,
             progress_listener=progress_listener,
             encryption=encryption,
             file_retention=file_retention,
             legal_hold=legal_hold,
             large_file_sha1=large_file_sha1,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def upload_local_file(
@@ -533,15 +536,16 @@ class Bucket(metaclass=B2TraceMeta):
         local_file,
         file_name,
         content_type=None,
-        file_infos=None,
+        file_info: dict | None = None,
         sha1_sum=None,
         min_part_size=None,
         progress_listener=None,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
         upload_mode: UploadMode = UploadMode.FULL,
-        custom_upload_timestamp: Optional[int] = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         """
         Upload a file on local disk to a B2 file.
@@ -556,7 +560,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param str local_file: a path to a file on local disk
         :param str file_name: a file name of the new B2 file
         :param str,None content_type: the MIME type, or ``None`` to accept the default based on file extension of the B2 file name
-        :param dict,None file_infos: a file info to store with the file or ``None`` to not store anything
+        :param dict,None file_info: a file info to store with the file or ``None`` to not store anything
         :param str,None sha1_sum: file SHA1 hash or ``None`` to compute it automatically
         :param int min_part_size: a minimum size of a part
         :param b2sdk.v2.AbstractProgressListener,None progress_listener: a progress listener object to use, or ``None`` to not report progress
@@ -565,6 +569,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param bool legal_hold: legal hold setting
         :param b2sdk.v2.UploadMode upload_mode: desired upload mode
         :param int,None custom_upload_timestamp: override object creation date, expressed as a number of milliseconds since epoch
+        :param str,None cache_control: an optional cache control setting. Syntax based on the section 14.9 of RFC 2616. Example string value: 'public, max-age=86400, s-maxage=3600, no-transform'.
         :rtype: b2sdk.v2.FileVersion
         """
         upload_source = UploadSourceLocalFile(local_path=local_file, content_sha1=sha1_sum)
@@ -588,7 +593,7 @@ class Bucket(metaclass=B2TraceMeta):
             sources,
             file_name,
             content_type=content_type,
-            file_info=file_infos,
+            file_info=file_info,
             min_part_size=min_part_size,
             progress_listener=progress_listener,
             encryption=encryption,
@@ -596,6 +601,7 @@ class Bucket(metaclass=B2TraceMeta):
             legal_hold=legal_hold,
             large_file_sha1=large_file_sha1,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def upload_unbound_stream(
@@ -603,20 +609,21 @@ class Bucket(metaclass=B2TraceMeta):
         read_only_object,
         file_name: str,
         content_type: str = None,
-        file_info: Optional[Dict[str, str]] = None,
-        progress_listener: Optional[AbstractProgressListener] = None,
-        recommended_upload_part_size: Optional[int] = None,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        min_part_size: Optional[int] = None,
-        max_part_size: Optional[int] = None,
-        large_file_sha1: Optional[Sha1HexDigest] = None,
+        file_info: dict[str, str] | None = None,
+        progress_listener: AbstractProgressListener | None = None,
+        recommended_upload_part_size: int | None = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        min_part_size: int | None = None,
+        max_part_size: int | None = None,
+        large_file_sha1: Sha1HexDigest | None = None,
         buffers_count: int = 2,
-        buffer_size: Optional[int] = None,
+        buffer_size: int | None = None,
         read_size: int = 8192,
         unused_buffer_timeout_seconds: float = 3600.0,
-        custom_upload_timestamp: Optional[int] = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         """
         Upload an unbound file-like read-only object to a B2 file.
@@ -680,6 +687,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param read_size: size of a single read operation performed on the ``read_only_object``
         :param unused_buffer_timeout_seconds: amount of time that a buffer can be idle before returning error
         :param int,None custom_upload_timestamp: override object creation date, expressed as a number of milliseconds since epoch
+        :param str,None cache_control: an optional cache control setting. Syntax based on the section 14.9 of RFC 2616. Example string value: 'public, max-age=86400, s-maxage=3600, no-transform'.
         :rtype: b2sdk.v2.FileVersion
         """
         if buffers_count <= 1:
@@ -719,6 +727,7 @@ class Bucket(metaclass=B2TraceMeta):
             max_queue_size=buffers_count - 1,
             large_file_sha1=large_file_sha1,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def upload(
@@ -729,11 +738,12 @@ class Bucket(metaclass=B2TraceMeta):
         file_info=None,
         min_part_size=None,
         progress_listener=None,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        large_file_sha1: Optional[Sha1HexDigest] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        large_file_sha1: Sha1HexDigest | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         """
         Upload a file to B2, retrying as needed.
@@ -760,6 +770,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param bool legal_hold: legal hold setting
         :param Sha1HexDigest,None large_file_sha1: SHA-1 hash of the result file or ``None`` if unknown
         :param int,None custom_upload_timestamp: override object creation date, expressed as a number of milliseconds since epoch
+        :param str,None cache_control: an optional cache control setting. Syntax based on the section 14.9 of RFC 2616. Example string value: 'public, max-age=86400, s-maxage=3600, no-transform'.
         :rtype: b2sdk.v2.FileVersion
         """
         return self.create_file(
@@ -775,6 +786,7 @@ class Bucket(metaclass=B2TraceMeta):
             legal_hold=legal_hold,
             large_file_sha1=large_file_sha1,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def create_file(
@@ -786,13 +798,14 @@ class Bucket(metaclass=B2TraceMeta):
         progress_listener=None,
         recommended_upload_part_size=None,
         continue_large_file_id=None,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
         min_part_size=None,
         max_part_size=None,
         large_file_sha1=None,
-        custom_upload_timestamp: Optional[int] = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         """
         Creates a new file in this bucket using an iterable (list, tuple etc) of remote or local sources.
@@ -824,6 +837,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param int max_part_size: upper limit of part size for the transfer planner, in bytes
         :param Sha1HexDigest,None large_file_sha1: SHA-1 hash of the result file or ``None`` if unknown
         :param int,None custom_upload_timestamp: override object creation date, expressed as a number of milliseconds since epoch
+        :param str,None cache_control: an optional cache control setting. Syntax based on the section 14.9 of RFC 2616. Example string value: 'public, max-age=86400, s-maxage=3600, no-transform'.
         """
         return self._create_file(
             self.api.services.emerger.emerge,
@@ -841,6 +855,7 @@ class Bucket(metaclass=B2TraceMeta):
             max_part_size=max_part_size,
             large_file_sha1=large_file_sha1,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def create_file_stream(
@@ -852,13 +867,14 @@ class Bucket(metaclass=B2TraceMeta):
         progress_listener=None,
         recommended_upload_part_size=None,
         continue_large_file_id=None,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
         min_part_size=None,
         max_part_size=None,
         large_file_sha1=None,
-        custom_upload_timestamp: Optional[int] = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         """
         Creates a new file in this bucket using a stream of multiple remote or local sources.
@@ -892,6 +908,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param int max_part_size: upper limit of part size for the transfer planner, in bytes
         :param Sha1HexDigest,None large_file_sha1: SHA-1 hash of the result file or ``None`` if unknown
         :param int,None custom_upload_timestamp: override object creation date, expressed as a number of milliseconds since epoch
+        :param str,None cache_control: an optional cache control setting. Syntax based on the section 14.9 of RFC 2616. Example string value: 'public, max-age=86400, s-maxage=3600, no-transform'.
         """
         return self._create_file(
             self.api.services.emerger.emerge_stream,
@@ -909,6 +926,7 @@ class Bucket(metaclass=B2TraceMeta):
             max_part_size=max_part_size,
             large_file_sha1=large_file_sha1,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def _create_file(
@@ -921,9 +939,9 @@ class Bucket(metaclass=B2TraceMeta):
         progress_listener=None,
         recommended_upload_part_size=None,
         continue_large_file_id=None,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
         min_part_size=None,
         max_part_size=None,
         large_file_sha1=None,
@@ -959,13 +977,14 @@ class Bucket(metaclass=B2TraceMeta):
         progress_listener=None,
         recommended_upload_part_size=None,
         continue_large_file_id=None,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
         min_part_size=None,
         max_part_size=None,
         large_file_sha1=None,
-        custom_upload_timestamp: Optional[int] = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         """
         Creates a new file in this bucket by concatenating multiple remote or local sources.
@@ -994,6 +1013,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param int max_part_size: upper limit of part size for the transfer planner, in bytes
         :param Sha1HexDigest,None large_file_sha1: SHA-1 hash of the result file or ``None`` if unknown
         :param int,None custom_upload_timestamp: override object creation date, expressed as a number of milliseconds since epoch
+        :param str,None cache_control: an optional cache control setting. Syntax based on the section 14.9 of RFC 2616. Example string value: 'public, max-age=86400, s-maxage=3600, no-transform'.
         """
         return self.create_file(
             list(WriteIntent.wrap_sources_iterator(outbound_sources)),
@@ -1010,6 +1030,7 @@ class Bucket(metaclass=B2TraceMeta):
             max_part_size=max_part_size,
             large_file_sha1=large_file_sha1,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def concatenate_stream(
@@ -1021,11 +1042,12 @@ class Bucket(metaclass=B2TraceMeta):
         progress_listener=None,
         recommended_upload_part_size=None,
         continue_large_file_id=None,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        large_file_sha1: Optional[Sha1HexDigest] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        large_file_sha1: Sha1HexDigest | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         """
         Creates a new file in this bucket by concatenating stream of multiple remote or local sources.
@@ -1050,6 +1072,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param bool legal_hold: legal hold setting
         :param Sha1HexDigest,None large_file_sha1: SHA-1 hash of the result file or ``None`` if unknown
         :param int,None custom_upload_timestamp: override object creation date, expressed as a number of milliseconds since epoch
+        :param str,None cache_control: an optional cache control setting. Syntax based on the section 14.9 of RFC 2616. Example string value: 'public, max-age=86400, s-maxage=3600, no-transform'.
         """
         return self.create_file_stream(
             WriteIntent.wrap_sources_iterator(outbound_sources_iterator),
@@ -1064,6 +1087,7 @@ class Bucket(metaclass=B2TraceMeta):
             legal_hold=legal_hold,
             large_file_sha1=large_file_sha1,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def get_download_url(self, filename):
@@ -1073,7 +1097,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param str filename: a file name
         :rtype: str
         """
-        return "%s/file/%s/%s" % (
+        return "{}/file/{}/{}".format(
             self.api.account_info.get_download_url(),
             b2_url_encode(self.name),
             b2_url_encode(filename),
@@ -1098,12 +1122,13 @@ class Bucket(metaclass=B2TraceMeta):
         offset=0,
         length=None,
         progress_listener=None,
-        destination_encryption: Optional[EncryptionSetting] = None,
-        source_encryption: Optional[EncryptionSetting] = None,
-        source_file_info: Optional[dict] = None,
-        source_content_type: Optional[str] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        destination_encryption: EncryptionSetting | None = None,
+        source_encryption: EncryptionSetting | None = None,
+        source_file_info: dict | None = None,
+        source_content_type: str | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        cache_control: str | None = None,
         min_part_size=None,
         max_part_size=None,
     ):
@@ -1131,6 +1156,7 @@ class Bucket(metaclass=B2TraceMeta):
         :param str,None source_content_type: source file's content type, useful when copying files with SSE-C
         :param b2sdk.v2.FileRetentionSetting file_retention: file retention setting for the new file.
         :param bool legal_hold: legal hold setting for the new file.
+        :param str,None cache_control: an optional cache control setting. Syntax based on the section 14.9 of RFC 2616. Example string value: 'public, max-age=86400, s-maxage=3600, no-transform'.
         :param int min_part_size: lower limit of part size for the transfer planner, in bytes
         :param int max_part_size: upper limit of part size for the transfer planner, in bytes
         """
@@ -1159,6 +1185,7 @@ class Bucket(metaclass=B2TraceMeta):
                     source_encryption=source_encryption,
                     file_retention=file_retention,
                     legal_hold=legal_hold,
+                    cache_control=cache_control,
                 ).result()
             except CopySourceTooBig as e:
                 copy_source.length = e.size
@@ -1175,6 +1202,7 @@ class Bucket(metaclass=B2TraceMeta):
             encryption=destination_encryption,
             file_retention=file_retention,
             legal_hold=legal_hold,
+            cache_control=cache_control,
             min_part_size=min_part_size,
             max_part_size=max_part_size,
         )
@@ -1217,7 +1245,7 @@ class Bucket(metaclass=B2TraceMeta):
         return result
 
     def __repr__(self):
-        return 'Bucket<%s,%s,%s>' % (self.id_, self.name, self.type_)
+        return f'Bucket<{self.id_},{self.name},{self.type_}>'
 
 
 class BucketFactory:
