@@ -7,17 +7,18 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
 
-from typing import Any, Dict, Optional, Union, Tuple, TYPE_CHECKING
 import re
 from copy import deepcopy
+from typing import TYPE_CHECKING, Any
 
 from .encryption.setting import EncryptionSetting, EncryptionSettingFactory
-from .file_lock import FileRetentionSetting, LegalHold, NO_RETENTION_FILE_SETTING
+from .file_lock import NO_RETENTION_FILE_SETTING, FileRetentionSetting, LegalHold
 from .http_constants import FILE_INFO_HEADER_PREFIX_LOWER, LARGE_FILE_SHA1, SRC_LAST_MODIFIED_MILLIS
 from .progress import AbstractProgressListener
 from .replication.types import ReplicationStatus
-from .utils import b2_url_decode, Sha1HexDigest
+from .utils import Sha1HexDigest, b2_url_decode
 from .utils.range_ import Range
 
 if TYPE_CHECKING:
@@ -48,6 +49,7 @@ class BaseFileVersion:
         'file_retention',
         'mod_time_millis',
         'replication_status',
+        'cache_control',
     ]
     _TYPE_MATCHER = re.compile('[a-z0-9]+_[a-z0-9]+_f([0-9]).*')
     _FILE_TYPE = {
@@ -59,18 +61,19 @@ class BaseFileVersion:
 
     def __init__(
         self,
-        api: 'B2Api',
+        api: B2Api,
         id_: str,
         file_name: str,
         size: int,
-        content_type: Optional[str],
-        content_sha1: Optional[str],
-        file_info: Dict[str, str],
+        content_type: str | None,
+        content_sha1: str | None,
+        file_info: dict[str, str] | None,
         upload_timestamp: int,
         server_side_encryption: EncryptionSetting,
         file_retention: FileRetentionSetting = NO_RETENTION_FILE_SETTING,
         legal_hold: LegalHold = LegalHold.UNSET,
-        replication_status: Optional[ReplicationStatus] = None,
+        replication_status: ReplicationStatus | None = None,
+        cache_control: str | None = None,
     ):
         self.api = api
         self.id_ = id_
@@ -84,6 +87,7 @@ class BaseFileVersion:
         self.file_retention = file_retention
         self.legal_hold = legal_hold
         self.replication_status = replication_status
+        self.cache_control = cache_control
 
         if SRC_LAST_MODIFIED_MILLIS in self.file_info:
             self.mod_time_millis = int(self.file_info[SRC_LAST_MODIFIED_MILLIS])
@@ -99,7 +103,7 @@ class BaseFileVersion:
     @classmethod
     def _encode_content_sha1(cls, content_sha1, content_sha1_verified):
         if not content_sha1_verified:
-            return '%s%s' % (UNVERIFIED_CHECKSUM_PREFIX, content_sha1)
+            return f'{UNVERIFIED_CHECKSUM_PREFIX}{content_sha1}'
         return content_sha1
 
     def _clone(self, **new_attributes: Any):
@@ -124,6 +128,7 @@ class BaseFileVersion:
             'file_retention': self.file_retention,
             'legal_hold': self.legal_hold,
             'replication_status': self.replication_status,
+            'cache_control': self.cache_control,
         }  # yapf: disable
 
     def as_dict(self):
@@ -135,6 +140,7 @@ class BaseFileVersion:
             'serverSideEncryption': self.server_side_encryption.as_dict(),
             'legalHold': self.legal_hold.value,
             'fileRetention': self.file_retention.as_dict(),
+            'cacheControl': self.cache_control,
         }
 
         if self.size is not None:
@@ -159,7 +165,7 @@ class BaseFileVersion:
         return True
 
     def __repr__(self):
-        return '%s(%s)' % (
+        return '{}({})'.format(
             self.__class__.__name__,
             ', '.join(repr(getattr(self, attr)) for attr in self._all_slots())
         )
@@ -171,10 +177,10 @@ class BaseFileVersion:
             all_slots.extend(getattr(klass, '__slots__', []))
         return all_slots
 
-    def delete(self) -> 'FileIdAndName':
+    def delete(self) -> FileIdAndName:
         return self.api.delete_file_version(self.id_, self.file_name)
 
-    def update_legal_hold(self, legal_hold: LegalHold) -> 'BaseFileVersion':
+    def update_legal_hold(self, legal_hold: LegalHold) -> BaseFileVersion:
         legal_hold = self.api.update_file_legal_hold(self.id_, self.file_name, legal_hold)
         return self._clone(legal_hold=legal_hold)
 
@@ -182,7 +188,7 @@ class BaseFileVersion:
         self,
         file_retention: FileRetentionSetting,
         bypass_governance: bool = False,
-    ) -> 'BaseFileVersion':
+    ) -> BaseFileVersion:
         file_retention = self.api.update_file_retention(
             self.id_, self.file_name, file_retention, bypass_governance
         )
@@ -197,7 +203,7 @@ class BaseFileVersion:
         assert m, self.id_
         return self._FILE_TYPE[int(m.group(1))]
 
-    def get_content_sha1(self) -> Optional[Sha1HexDigest]:
+    def get_content_sha1(self) -> Sha1HexDigest | None:
         """
         Get the file's content SHA1 hex digest from the header or, if its absent,
         from the file info.  If both are missing, return None.
@@ -235,22 +241,23 @@ class FileVersion(BaseFileVersion):
 
     def __init__(
         self,
-        api: 'B2Api',
+        api: B2Api,
         id_: str,
         file_name: str,
-        size: Union[int, None, str],
-        content_type: Optional[str],
-        content_sha1: Optional[str],
-        file_info: Dict[str, str],
+        size: int | None | str,
+        content_type: str | None,
+        content_sha1: str | None,
+        file_info: dict[str, str],
         upload_timestamp: int,
         account_id: str,
         bucket_id: str,
         action: str,
-        content_md5: Optional[str],
+        content_md5: str | None,
         server_side_encryption: EncryptionSetting,
         file_retention: FileRetentionSetting = NO_RETENTION_FILE_SETTING,
         legal_hold: LegalHold = LegalHold.UNSET,
-        replication_status: Optional[ReplicationStatus] = None,
+        replication_status: ReplicationStatus | None = None,
+        cache_control: str | None = None,
     ):
         self.account_id = account_id
         self.bucket_id = bucket_id
@@ -270,6 +277,7 @@ class FileVersion(BaseFileVersion):
             file_retention=file_retention,
             legal_hold=legal_hold,
             replication_status=replication_status,
+            cache_control=cache_control,
         )
 
     def _get_args_for_clone(self):
@@ -296,7 +304,7 @@ class FileVersion(BaseFileVersion):
 
         return result
 
-    def get_fresh_state(self) -> 'FileVersion':
+    def get_fresh_state(self) -> FileVersion:
         """
         Fetch all the information about this file version and return a new FileVersion object.
         This method does NOT change the object it is called on.
@@ -305,10 +313,10 @@ class FileVersion(BaseFileVersion):
 
     def download(
         self,
-        progress_listener: Optional[AbstractProgressListener] = None,
-        range_: Optional[Tuple[int, int]] = None,
-        encryption: Optional[EncryptionSetting] = None,
-    ) -> 'DownloadedFile':
+        progress_listener: AbstractProgressListener | None = None,
+        range_: tuple[int, int] | None = None,
+        encryption: EncryptionSetting | None = None,
+    ) -> DownloadedFile:
         return self.api.download_file_by_id(
             self.id_,
             progress_listener=progress_listener,
@@ -338,10 +346,11 @@ class FileVersion(BaseFileVersion):
             content_length=self.size,
             content_type=self.content_type,
             content_sha1=self.content_sha1,
-            file_infos=self.file_info,
+            file_info=self.file_info,
             server_side_encryption=sse,
             file_retention=self.file_retention,
             legal_hold=self.legal_hold,
+            cache_control=self.cache_control,
         )
 
         headers_str = ''.join(
@@ -377,25 +386,25 @@ class DownloadVersion(BaseFileVersion):
 
     def __init__(
         self,
-        api: 'B2Api',
+        api: B2Api,
         id_: str,
         file_name: str,
         size: int,
-        content_type: Optional[str],
-        content_sha1: Optional[str],
-        file_info: Dict[str, str],
+        content_type: str | None,
+        content_sha1: str | None,
+        file_info: dict[str, str],
         upload_timestamp: int,
         server_side_encryption: EncryptionSetting,
         range_: Range,
-        content_disposition: Optional[str],
+        content_disposition: str | None,
         content_length: int,
-        content_language: Optional[str],
+        content_language: str | None,
         expires,
         cache_control,
-        content_encoding: Optional[str],
+        content_encoding: str | None,
         file_retention: FileRetentionSetting = NO_RETENTION_FILE_SETTING,
         legal_hold: LegalHold = LegalHold.UNSET,
-        replication_status: Optional[ReplicationStatus] = None,
+        replication_status: ReplicationStatus | None = None,
     ):
         self.range_ = range_
         self.content_disposition = content_disposition
@@ -418,6 +427,7 @@ class DownloadVersion(BaseFileVersion):
             file_retention=file_retention,
             legal_hold=legal_hold,
             replication_status=replication_status,
+            cache_control=cache_control,
         )
 
     def _get_args_for_clone(self):
@@ -443,7 +453,7 @@ class FileVersionFactory:
 
     FILE_VERSION_CLASS = FileVersion
 
-    def __init__(self, api: 'B2Api'):
+    def __init__(self, api: B2Api):
         self.api = api
 
     def from_api_response(self, file_version_dict, force_action=None):
@@ -501,10 +511,10 @@ class FileVersionFactory:
         file_retention = FileRetentionSetting.from_file_version_dict(file_version_dict)
 
         legal_hold = LegalHold.from_file_version_dict(file_version_dict)
-
         replication_status_value = file_version_dict.get('replicationStatus')
         replication_status = replication_status_value and ReplicationStatus[
             replication_status_value.upper()]
+        cache_control = file_version_dict.get('cacheControl')
 
         return self.FILE_VERSION_CLASS(
             self.api,
@@ -523,6 +533,7 @@ class FileVersionFactory:
             file_retention,
             legal_hold,
             replication_status,
+            cache_control,
         )
 
 
@@ -531,7 +542,7 @@ class DownloadVersionFactory:
     Construct :py:class:`b2sdk.v2.DownloadVersion` objects from download headers.
     """
 
-    def __init__(self, api: 'B2Api'):
+    def __init__(self, api: B2Api):
         self.api = api
 
     @classmethod
@@ -554,6 +565,7 @@ class DownloadVersionFactory:
 
     def from_response_headers(self, headers):
         file_info = self.file_info_from_headers(headers)
+
         if 'Content-Range' in headers:
             range_, size = self.range_and_size_from_header(headers['Content-Range'])
             content_length = int(headers['Content-Length'])
@@ -561,6 +573,11 @@ class DownloadVersionFactory:
             size = content_length = int(headers['Content-Length'])
             range_ = Range(0, max(size - 1, 0))
 
+        if 'Cache-Control' in headers:
+            cache_control = b2_url_decode(headers['Cache-Control'])
+        else:
+            cache_control = None
+
         return DownloadVersion(
             api=self.api,
             id_=headers['x-bz-file-id'],
@@ -576,7 +593,7 @@ class DownloadVersionFactory:
             content_length=content_length,
             content_language=headers.get('Content-Language'),
             expires=headers.get('Expires'),
-            cache_control=headers.get('Cache-Control'),
+            cache_control=cache_control,
             content_encoding=headers.get('Content-Encoding'),
             file_retention=FileRetentionSetting.from_response_headers(headers),
             legal_hold=LegalHold.from_response_headers(headers),
@@ -607,4 +624,4 @@ class FileIdAndName:
         return (self.file_id == other.file_id and self.file_name == other.file_name)
 
     def __repr__(self):
-        return '%s(%s, %s)' % (self.__class__.__name__, repr(self.file_id), repr(self.file_name))
+        return f'{self.__class__.__name__}({repr(self.file_id)}, {repr(self.file_name)})'
