@@ -7,20 +7,37 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
 
 import base64
 import re
 from abc import ABCMeta, abstractmethod
 from enum import Enum, unique
 from logging import getLogger
-from typing import Any, Dict, Optional
+from typing import Any
+
+try:
+    from typing import NotRequired, TypedDict
+except ImportError:
+    from typing_extensions import NotRequired, TypedDict
 
-from .exception import FileOrBucketNotFound, ResourceNotFound, UnusableFileName, InvalidMetadataDirective, WrongEncryptionModeForBucketDefault, AccessDenied, SSECKeyError, RetentionWriteError
-from .encryption.setting import EncryptionMode, EncryptionSetting
-from .replication.setting import ReplicationConfiguration
-from .file_lock import BucketRetentionSetting, FileRetentionSetting, LegalHold
-from .utils import b2_url_encode
 from b2sdk.http_constants import FILE_INFO_HEADER_PREFIX
+from b2sdk.utils.docs import ensure_b2sdk_doc_urls
+
+from .encryption.setting import EncryptionMode, EncryptionSetting
+from .exception import (
+    AccessDenied,
+    FileOrBucketNotFound,
+    InvalidMetadataDirective,
+    ResourceNotFound,
+    RetentionWriteError,
+    SSECKeyError,
+    UnusableFileName,
+    WrongEncryptionModeForBucketDefault,
+)
+from .file_lock import BucketRetentionSetting, FileRetentionSetting, LegalHold
+from .replication.setting import ReplicationConfiguration
+from .utils import b2_url_encode
 
 # All supported realms
 REALM_URLS = {
@@ -70,6 +87,20 @@ class MetadataDirectiveMode(Enum):
     REPLACE = 402  #: ignore the source file metadata and set it to provided values
 
 
+@ensure_b2sdk_doc_urls
+class LifecycleRule(TypedDict):
+    """
+    Lifecycle Rule.
+
+    External documentation: `B2 Cloud Storage Lifecycle Rules`_.
+
+    .. _B2 Cloud Storage Lifecycle Rules: https://www.backblaze.com/docs/cloud-storage-lifecycle-rules
+    """
+    fileNamePrefix: str
+    daysFromHidingToDeleting: NotRequired[int | None]
+    daysFromUploadingToHiding: NotRequired[int | None]
+
+
 class AbstractRawApi(metaclass=ABCMeta):
     """
     Direct access to the B2 web apis.
@@ -95,10 +126,11 @@ class AbstractRawApi(metaclass=ABCMeta):
         content_type=None,
         file_info=None,
         destination_bucket_id=None,
-        destination_server_side_encryption: Optional[EncryptionSetting] = None,
-        source_server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        destination_server_side_encryption: EncryptionSetting | None = None,
+        source_server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        cache_control: str | None = None,
     ):
         pass
 
@@ -111,8 +143,8 @@ class AbstractRawApi(metaclass=ABCMeta):
         large_file_id,
         part_number,
         bytes_range=None,
-        destination_server_side_encryption: Optional[EncryptionSetting] = None,
-        source_server_side_encryption: Optional[EncryptionSetting] = None,
+        destination_server_side_encryption: EncryptionSetting | None = None,
+        source_server_side_encryption: EncryptionSetting | None = None,
     ):
         pass
 
@@ -126,10 +158,10 @@ class AbstractRawApi(metaclass=ABCMeta):
         bucket_type,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
-        default_server_side_encryption: Optional[EncryptionSetting] = None,
-        is_file_lock_enabled: Optional[bool] = None,
-        replication: Optional[ReplicationConfiguration] = None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
+        default_server_side_encryption: EncryptionSetting | None = None,
+        is_file_lock_enabled: bool | None = None,
+        replication: ReplicationConfiguration | None = None,
     ):
         pass
 
@@ -146,7 +178,7 @@ class AbstractRawApi(metaclass=ABCMeta):
         account_auth_token_or_none,
         url,
         range_=None,
-        encryption: Optional[EncryptionSetting] = None,
+        encryption: EncryptionSetting | None = None,
     ):
         pass
 
@@ -174,13 +206,13 @@ class AbstractRawApi(metaclass=ABCMeta):
 
     @abstractmethod
     def get_file_info_by_id(self, api_url: str, account_auth_token: str,
-                            file_id: str) -> Dict[str, Any]:
+                            file_id: str) -> dict[str, Any]:
         pass
 
     @abstractmethod
     def get_file_info_by_name(
         self, download_url: str, account_auth_token: str, bucket_name: str, file_name: str
-    ) -> Dict[str, Any]:
+    ) -> dict[str, Any]:
         pass
 
     @abstractmethod
@@ -267,9 +299,10 @@ class AbstractRawApi(metaclass=ABCMeta):
         file_name,
         content_type,
         file_info,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        cache_control: str | None = None,
     ):
         pass
 
@@ -283,12 +316,12 @@ class AbstractRawApi(metaclass=ABCMeta):
         bucket_type=None,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
         if_revision_is=None,
-        default_server_side_encryption: Optional[EncryptionSetting] = None,
-        default_retention: Optional[BucketRetentionSetting] = None,
-        replication: Optional[ReplicationConfiguration] = None,
-        is_file_lock_enabled: Optional[bool] = None,
+        default_server_side_encryption: EncryptionSetting | None = None,
+        default_retention: BucketRetentionSetting | None = None,
+        replication: ReplicationConfiguration | None = None,
+        is_file_lock_enabled: bool | None = None,
     ):
         pass
 
@@ -312,11 +345,12 @@ class AbstractRawApi(metaclass=ABCMeta):
         content_length: int,
         content_type: str,
         content_sha1: str,
-        file_infos: dict,
-        server_side_encryption: Optional[EncryptionSetting],
-        file_retention: Optional[FileRetentionSetting],
-        legal_hold: Optional[LegalHold],
-        custom_upload_timestamp: Optional[int] = None,
+        file_info: dict,
+        server_side_encryption: EncryptionSetting | None,
+        file_retention: FileRetentionSetting | None,
+        legal_hold: LegalHold | None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ) -> dict:
         headers = {
             'Authorization': upload_auth_token,
@@ -325,7 +359,7 @@ class AbstractRawApi(metaclass=ABCMeta):
             'Content-Type': content_type,
             'X-Bz-Content-Sha1': content_sha1,
         }
-        for k, v in file_infos.items():
+        for k, v in file_info.items():
             headers[FILE_INFO_HEADER_PREFIX + k] = b2_url_encode(v)
         if server_side_encryption is not None:
             assert server_side_encryption.mode in (
@@ -339,6 +373,9 @@ class AbstractRawApi(metaclass=ABCMeta):
         if file_retention is not None:
             file_retention.add_to_to_upload_headers(headers)
 
+        if cache_control is not None:
+            headers['X-Bz-Info-b2-cache-control'] = b2_url_encode(cache_control)
+
         if custom_upload_timestamp is not None:
             headers['X-Bz-Custom-Upload-Timestamp'] = str(custom_upload_timestamp)
 
@@ -353,12 +390,13 @@ class AbstractRawApi(metaclass=ABCMeta):
         content_length,
         content_type,
         content_sha1,
-        file_infos,
+        file_info,
         data_stream,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         pass
 
@@ -371,12 +409,12 @@ class AbstractRawApi(metaclass=ABCMeta):
         content_length,
         sha1_sum,
         input_stream,
-        server_side_encryption: Optional[EncryptionSetting] = None,
+        server_side_encryption: EncryptionSetting | None = None,
     ):
         pass
 
     def get_download_url_by_id(self, download_url, file_id):
-        return '%s/b2api/%s/b2_download_file_by_id?fileId=%s' % (download_url, API_VERSION, file_id)
+        return f'{download_url}/b2api/{API_VERSION}/b2_download_file_by_id?fileId={file_id}'
 
     def get_download_url_by_name(self, download_url, bucket_name, file_name):
         return download_url + '/file/' + bucket_name + '/' + b2_url_encode(file_name)
@@ -402,7 +440,7 @@ class B2RawHTTPApi(AbstractRawApi):
     def __init__(self, b2_http):
         self.b2_http = b2_http
 
-    def _post_json(self, base_url, api_name, auth, **params) -> Dict[str, Any]:
+    def _post_json(self, base_url, api_name, auth, **params) -> dict[str, Any]:
         """
         A helper method for calling an API with the given auth and params.
 
@@ -413,14 +451,12 @@ class B2RawHTTPApi(AbstractRawApi):
         :return: the decoded JSON response
         :rtype: dict
         """
-        url = '%s/b2api/%s/%s' % (base_url, API_VERSION, api_name)
+        url = f'{base_url}/b2api/{API_VERSION}/{api_name}'
         headers = {'Authorization': auth}
         return self.b2_http.post_json_return_json(url, headers, params)
 
     def authorize_account(self, realm_url, application_key_id, application_key):
-        auth = b'Basic ' + base64.b64encode(
-            ('%s:%s' % (application_key_id, application_key)).encode()
-        )
+        auth = b'Basic ' + base64.b64encode((f'{application_key_id}:{application_key}').encode())
         return self._post_json(realm_url, 'b2_authorize_account', auth)
 
     def cancel_large_file(self, api_url, account_auth_token, file_id):
@@ -435,10 +471,10 @@ class B2RawHTTPApi(AbstractRawApi):
         bucket_type,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
-        default_server_side_encryption: Optional[EncryptionSetting] = None,
-        is_file_lock_enabled: Optional[bool] = None,
-        replication: Optional[ReplicationConfiguration] = None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
+        default_server_side_encryption: EncryptionSetting | None = None,
+        is_file_lock_enabled: bool | None = None,
+        replication: ReplicationConfiguration | None = None,
     ):
         kwargs = dict(
             accountId=account_id,
@@ -514,7 +550,7 @@ class B2RawHTTPApi(AbstractRawApi):
         account_auth_token_or_none,
         url,
         range_=None,
-        encryption: Optional[EncryptionSetting] = None,
+        encryption: EncryptionSetting | None = None,
     ):
         """
         Issue a streaming request for download of a file, potentially authorized.
@@ -563,12 +599,12 @@ class B2RawHTTPApi(AbstractRawApi):
         )
 
     def get_file_info_by_id(self, api_url: str, account_auth_token: str,
-                            file_id: str) -> Dict[str, Any]:
+                            file_id: str) -> dict[str, Any]:
         return self._post_json(api_url, 'b2_get_file_info', account_auth_token, fileId=file_id)
 
     def get_file_info_by_name(
         self, download_url: str, account_auth_token: str, bucket_name: str, file_name: str
-    ) -> Dict[str, Any]:
+    ) -> dict[str, Any]:
         download_url = self.get_download_url_by_name(download_url, bucket_name, file_name)
         try:
             response = self.b2_http.head_content(
@@ -704,10 +740,11 @@ class B2RawHTTPApi(AbstractRawApi):
         file_name,
         content_type,
         file_info,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         kwargs = {}
         if server_side_encryption is not None:
@@ -725,6 +762,9 @@ class B2RawHTTPApi(AbstractRawApi):
         if custom_upload_timestamp is not None:
             kwargs['custom_upload_timestamp'] = custom_upload_timestamp
 
+        if cache_control is not None:
+            kwargs['cacheControl'] = cache_control
+
         return self._post_json(
             api_url,
             'b2_start_large_file',
@@ -745,12 +785,12 @@ class B2RawHTTPApi(AbstractRawApi):
         bucket_type=None,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
         if_revision_is=None,
-        default_server_side_encryption: Optional[EncryptionSetting] = None,
-        default_retention: Optional[BucketRetentionSetting] = None,
-        replication: Optional[ReplicationConfiguration] = None,
-        is_file_lock_enabled: Optional[bool] = None,
+        default_server_side_encryption: EncryptionSetting | None = None,
+        default_retention: BucketRetentionSetting | None = None,
+        replication: ReplicationConfiguration | None = None,
+        is_file_lock_enabled: bool | None = None,
     ):
         kwargs = {}
         if if_revision_is is not None:
@@ -841,7 +881,7 @@ class B2RawHTTPApi(AbstractRawApi):
         unprintables_pattern = re.compile(r'[\x00-\x1f]')
 
         def hexify(match):
-            return r'\x{0:02x}'.format(ord(match.group()))
+            return fr'\x{ord(match.group()):02x}'
 
         return unprintables_pattern.sub(hexify, string)
 
@@ -862,7 +902,7 @@ class B2RawHTTPApi(AbstractRawApi):
             raise UnusableFileName("Filename is too long (can be at most 1024 bytes).")
         lowest_unicode_value = ord(min(filename))
         if lowest_unicode_value < 32:
-            message = u"Filename \"{0}\" contains code {1} (hex {2:02x}), less than 32.".format(
+            message = "Filename \"{}\" contains code {} (hex {:02x}), less than 32.".format(
                 self.unprintable_to_hex(filename), lowest_unicode_value, lowest_unicode_value
             )
             raise UnusableFileName(message)
@@ -885,12 +925,13 @@ class B2RawHTTPApi(AbstractRawApi):
         content_length,
         content_type,
         content_sha1,
-        file_infos,
+        file_info: dict,
         data_stream,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         """
         Upload one, small file to b2.
@@ -901,8 +942,13 @@ class B2RawHTTPApi(AbstractRawApi):
         :param content_length: number of bytes in the file
         :param content_type: MIME type
         :param content_sha1: hex SHA1 of the contents of the file
-        :param file_infos: extra file info to upload
+        :param file_info: extra file info to upload
         :param data_stream: a file like object from which the contents of the file can be read
+        :param server_side_encryption: encryption setting for the file
+        :param file_retention: retention setting for the file
+        :param legal_hold: legal hold setting for the file
+        :param custom_upload_timestamp: custom upload timestamp for the file
+        :param cache_control: an optional cache control setting. Syntax based on the section 14.9 of RFC 2616. Example string value: 'public, max-age=86400, s-maxage=3600, no-transform'.
         :return:
         """
         # Raise UnusableFileName if the file_name doesn't meet the rules.
@@ -913,11 +959,12 @@ class B2RawHTTPApi(AbstractRawApi):
             content_length=content_length,
             content_type=content_type,
             content_sha1=content_sha1,
-            file_infos=file_infos,
+            file_info=file_info,
             server_side_encryption=server_side_encryption,
             file_retention=file_retention,
             legal_hold=legal_hold,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
         return self.b2_http.post_content_return_json(upload_url, headers, data_stream)
 
@@ -929,7 +976,7 @@ class B2RawHTTPApi(AbstractRawApi):
         content_length,
         content_sha1,
         data_stream,
-        server_side_encryption: Optional[EncryptionSetting] = None,
+        server_side_encryption: EncryptionSetting | None = None,
     ):
         headers = {
             'Authorization': upload_auth_token,
@@ -956,10 +1003,11 @@ class B2RawHTTPApi(AbstractRawApi):
         content_type=None,
         file_info=None,
         destination_bucket_id=None,
-        destination_server_side_encryption: Optional[EncryptionSetting] = None,
-        source_server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        destination_server_side_encryption: EncryptionSetting | None = None,
+        source_server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        cache_control: str | None = None,
     ):
         kwargs = {}
         if bytes_range is not None:
@@ -1004,6 +1052,9 @@ class B2RawHTTPApi(AbstractRawApi):
         if file_retention is not None:
             kwargs['fileRetention'] = file_retention.serialize_to_json_for_request()
 
+        if cache_control is not None:
+            kwargs['cacheControl'] = cache_control
+
         try:
             return self._post_json(
                 api_url,
@@ -1024,8 +1075,8 @@ class B2RawHTTPApi(AbstractRawApi):
         large_file_id,
         part_number,
         bytes_range=None,
-        destination_server_side_encryption: Optional[EncryptionSetting] = None,
-        source_server_side_encryption: Optional[EncryptionSetting] = None,
+        destination_server_side_encryption: EncryptionSetting | None = None,
+        source_server_side_encryption: EncryptionSetting | None = None,
     ):
         kwargs = {}
         if bytes_range is not None:
