@@ -7,6 +7,7 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
 
 import collections
 import io
@@ -15,17 +16,15 @@ import random
 import re
 import threading
 import time
-
 from contextlib import contextmanager, suppress
-from typing import Optional
+
+from requests.structures import CaseInsensitiveDict
 
 from b2sdk.http_constants import FILE_INFO_HEADER_PREFIX, HEX_DIGITS_AT_END
 from b2sdk.replication.setting import ReplicationConfiguration
-from requests.structures import CaseInsensitiveDict
 
 from .b2http import ResponseContextManager
 from .encryption.setting import EncryptionMode, EncryptionSetting
-from .replication.types import ReplicationStatus
 from .exception import (
     BadJson,
     BadRequest,
@@ -42,8 +41,8 @@ from .exception import (
     MissingPart,
     NonExistentBucket,
     PartSha1Mismatch,
-    SSECKeyError,
     SourceReplicationConflict,
+    SSECKeyError,
     Unauthorized,
     UnsatisfiableRange,
 )
@@ -54,7 +53,8 @@ from .file_lock import (
     LegalHold,
 )
 from .file_version import UNVERIFIED_CHECKSUM_PREFIX
-from .raw_api import ALL_CAPABILITIES, AbstractRawApi, MetadataDirectiveMode
+from .raw_api import ALL_CAPABILITIES, AbstractRawApi, LifecycleRule, MetadataDirectiveMode
+from .replication.types import ReplicationStatus
 from .stream.hashing import StreamWithHash
 from .utils import ConcurrentUsedAuthTokenGuard, b2_url_decode, b2_url_encode, hex_sha1_of_bytes
 
@@ -172,10 +172,11 @@ class FileSimulator:
         data_bytes,
         upload_timestamp,
         range_=None,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
         legal_hold: LegalHold = LegalHold.UNSET,
-        replication_status: Optional[ReplicationStatus] = None,
+        replication_status: ReplicationStatus | None = None,
+        cache_control: str | None = None,
     ):
         if action == 'hide':
             assert server_side_encryption is None
@@ -200,6 +201,7 @@ class FileSimulator:
         self.file_retention = file_retention
         self.legal_hold = legal_hold if legal_hold is not None else LegalHold.UNSET
         self.replication_status = replication_status
+        self.cache_control = cache_control
 
         if action == 'start':
             self.parts = []
@@ -274,9 +276,7 @@ class FileSimulator:
             elif self.server_side_encryption.mode in (EncryptionMode.NONE, EncryptionMode.UNKNOWN):
                 pass
             else:
-                raise ValueError(
-                    'Unsupported encryption mode: %s' % (self.server_side_encryption.mode,)
-                )
+                raise ValueError(f'Unsupported encryption mode: {self.server_side_encryption.mode}')
 
         if range_ is not None:
             headers['Content-Range'] = 'bytes %d-%d/%d' % (
@@ -297,6 +297,7 @@ class FileSimulator:
             action=self.action,
             uploadTimestamp=self.upload_timestamp,
             replicationStatus=self.replication_status and self.replication_status.value,
+            cacheControl=self.cache_control,
         )  # yapf: disable
         if self.server_side_encryption is not None:
             result['serverSideEncryption'
@@ -318,6 +319,7 @@ class FileSimulator:
             action=self.action,
             uploadTimestamp=self.upload_timestamp,
             replicationStatus=self.replication_status and self.replication_status.value,
+            cacheControl=self.cache_control,
         )  # yapf: disable
         if self.server_side_encryption is not None:
             result['serverSideEncryption'
@@ -342,6 +344,7 @@ class FileSimulator:
             fileInfo=self.file_info,
             uploadTimestamp=self.upload_timestamp,
             replicationStatus=self.replication_status and self.replication_status.value,
+            cacheControl=self.cache_control,
         )  # yapf: disable
         if self.server_side_encryption is not None:
             result['serverSideEncryption'
@@ -421,7 +424,7 @@ class FileSimulator:
             parts = parts[:max_part_count]
         return dict(parts=parts, nextPartNumber=next_part_number)
 
-    def check_encryption(self, request_encryption: Optional[EncryptionSetting]):
+    def check_encryption(self, request_encryption: EncryptionSetting | None):
         if not self.CHECK_ENCRYPTION:
             return
         file_mode, file_secret = self._get_encryption_mode_and_secret(self.server_side_encryption)
@@ -437,7 +440,7 @@ class FileSimulator:
         else:
             raise ValueError('Unsupported EncryptionMode: %s' % (file_mode))
 
-    def _get_encryption_mode_and_secret(self, encryption: Optional[EncryptionSetting]):
+    def _get_encryption_mode_and_secret(self, encryption: EncryptionSetting | None):
         if encryption is None:
             return None, None
         mode = encryption.mode
@@ -472,7 +475,7 @@ class FakeResponse:
     def request(self):
         headers = CaseInsensitiveDict()
         if self.range_ is not None:
-            headers['Range'] = '%s-%s' % self.range_
+            headers['Range'] = '{}-{}'.format(*self.range_)
         return FakeRequest(self.url, headers)
 
     def close(self):
@@ -500,11 +503,11 @@ class BucketSimulator:
         bucket_type,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
         options_set=None,
         default_server_side_encryption=None,
-        is_file_lock_enabled: Optional[bool] = None,
-        replication: Optional[ReplicationConfiguration] = None,
+        is_file_lock_enabled: bool | None = None,
+        replication: ReplicationConfiguration | None = None,
     ):
         assert bucket_type in ['allPrivate', 'allPublic']
         self.api = api
@@ -622,7 +625,7 @@ class BucketSimulator:
         file_id,
         url,
         range_=None,
-        encryption: Optional[EncryptionSetting] = None,
+        encryption: EncryptionSetting | None = None,
     ):
         file_sim = self.file_id_to_file[file_id]
         file_sim.check_encryption(encryption)
@@ -634,7 +637,7 @@ class BucketSimulator:
         file_name,
         url,
         range_=None,
-        encryption: Optional[EncryptionSetting] = None,
+        encryption: EncryptionSetting | None = None,
     ):
         files = self.list_file_names(self.api.current_token, file_name,
                                      1)['files']  # token is not important here
@@ -746,10 +749,11 @@ class BucketSimulator:
         content_type=None,
         file_info=None,
         destination_bucket_id=None,
-        destination_server_side_encryption: Optional[EncryptionSetting] = None,
-        source_server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        destination_server_side_encryption: EncryptionSetting | None = None,
+        source_server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        cache_control: str | None = None,
     ):
         if metadata_directive is not None:
             assert metadata_directive in tuple(MetadataDirectiveMode), metadata_directive
@@ -792,6 +796,7 @@ class BucketSimulator:
             server_side_encryption=sse,
             file_retention=file_retention,
             legal_hold=legal_hold,
+            cache_control=cache_control,
         )  # yapf: disable
         destination_bucket.file_id_to_file[copy_file_sim.file_id] = copy_file_sim
         destination_bucket.file_name_and_id_to_file[copy_file_sim.sort_key()] = copy_file_sim
@@ -826,6 +831,7 @@ class BucketSimulator:
             server_side_encryption=sse,
             file_retention=file_retention,
             legal_hold=legal_hold,
+            cache_control=cache_control,
         )
 
     def list_file_names(
@@ -924,10 +930,11 @@ class BucketSimulator:
         file_name,
         content_type,
         file_info,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         file_id = self._next_file_id()
         sse = server_side_encryption or self.default_server_side_encryption
@@ -941,7 +948,7 @@ class BucketSimulator:
         file_sim = self.FILE_SIMULATOR_CLASS(
             self.account_id, self, file_id, 'start', file_name, content_type, 'none',
             file_info, None, upload_timestamp, server_side_encryption=sse,
-            file_retention=file_retention, legal_hold=legal_hold,
+            file_retention=file_retention, legal_hold=legal_hold, cache_control=cache_control,
         )  # yapf: disable
         self.file_id_to_file[file_id] = file_sim
         self.file_name_and_id_to_file[file_sim.sort_key()] = file_sim
@@ -952,12 +959,12 @@ class BucketSimulator:
         bucket_type=None,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
-        if_revision_is: Optional[int] = None,
-        default_server_side_encryption: Optional[EncryptionSetting] = None,
-        default_retention: Optional[BucketRetentionSetting] = None,
-        replication: Optional[ReplicationConfiguration] = None,
-        is_file_lock_enabled: Optional[bool] = None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
+        if_revision_is: int | None = None,
+        default_server_side_encryption: EncryptionSetting | None = None,
+        default_retention: BucketRetentionSetting | None = None,
+        replication: ReplicationConfiguration | None = None,
+        is_file_lock_enabled: bool | None = None,
     ):
         if if_revision_is is not None and self.revision != if_revision_is:
             raise Conflict()
@@ -1000,12 +1007,13 @@ class BucketSimulator:
         content_length: int,
         content_type: str,
         content_sha1: str,
-        file_infos: dict,
+        file_info: dict,
         data_stream,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         data_bytes = self._simulate_chunked_post(data_stream, content_length)
         assert len(data_bytes) == content_length
@@ -1024,7 +1032,7 @@ class BucketSimulator:
 
         encryption = server_side_encryption or self.default_server_side_encryption
         if encryption:  # FIXME: remove this part when RawApi<->Encryption adapters are implemented properly
-            file_infos = encryption.add_key_id_to_file_info(file_infos)
+            file_info = encryption.add_key_id_to_file_info(file_info)
 
         upload_timestamp = next(self.upload_timestamp_counter)
         if custom_upload_timestamp is not None:
@@ -1038,12 +1046,13 @@ class BucketSimulator:
             file_name,
             content_type,
             content_sha1,
-            file_infos,
+            file_info,
             data_bytes,
             upload_timestamp,
             server_side_encryption=encryption,
             file_retention=file_retention,
             legal_hold=legal_hold,
+            cache_control=cache_control,
         )
         self.file_id_to_file[file_id] = file_sim
         self.file_name_and_id_to_file[file_sim.sort_key()] = file_sim
@@ -1056,9 +1065,8 @@ class BucketSimulator:
         content_length,
         sha1_sum,
         input_stream,
-        server_side_encryption: Optional[EncryptionSetting] = None,
+        server_side_encryption: EncryptionSetting | None = None,
     ):
-        file_sim = self.file_id_to_file[file_id]
         part_data = self._simulate_chunked_post(input_stream, content_length)
         assert len(part_data) == content_length
         if sha1_sum == HEX_DIGITS_AT_END:
@@ -1068,8 +1076,11 @@ class BucketSimulator:
         computed_sha1 = hex_sha1_of_bytes(part_data)
         if sha1_sum != computed_sha1:
             raise PartSha1Mismatch(file_id)
+
+        file_sim = self.file_id_to_file[file_id]
         part = PartSimulator(file_sim.file_id, part_number, content_length, sha1_sum, part_data)
         file_sim.add_part(part_number, part)
+
         result = dict(
             fileId=file_id,
             partNumber=part_number,
@@ -1261,10 +1272,10 @@ class RawSimulator(AbstractRawApi):
         bucket_type,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
-        default_server_side_encryption: Optional[EncryptionSetting] = None,
-        is_file_lock_enabled: Optional[bool] = None,
-        replication: Optional[ReplicationConfiguration] = None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
+        default_server_side_encryption: EncryptionSetting | None = None,
+        is_file_lock_enabled: bool | None = None,
+        replication: ReplicationConfiguration | None = None,
     ):
         if not re.match(r'^[-a-zA-Z0-9]*$', bucket_name):
             raise BadJson('illegal bucket name: ' + bucket_name)
@@ -1395,7 +1406,7 @@ class RawSimulator(AbstractRawApi):
         account_auth_token_or_none,
         url,
         range_=None,
-        encryption: Optional[EncryptionSetting] = None
+        encryption: EncryptionSetting | None = None
     ):
         # TODO: check auth token if bucket is not public
         matcher = self.DOWNLOAD_URL_MATCHER.match(url)
@@ -1431,7 +1442,7 @@ class RawSimulator(AbstractRawApi):
         key_sim = self.key_id_to_key.pop(application_key_id, None)
         if key_sim is None:
             raise BadRequest(
-                'application key does not exist: %s' % (application_key_id,),
+                f'application key does not exist: {application_key_id}',
                 'bad_request',
             )
         self.all_application_keys = [
@@ -1504,8 +1515,9 @@ class RawSimulator(AbstractRawApi):
         destination_bucket_id=None,
         destination_server_side_encryption=None,
         source_server_side_encryption=None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        cache_control: str | None = None,
     ):
         bucket_id = self.file_id_to_bucket_id[source_file_id]
         bucket = self._get_bucket_by_id(bucket_id)
@@ -1531,6 +1543,7 @@ class RawSimulator(AbstractRawApi):
             source_server_side_encryption=source_server_side_encryption,
             file_retention=file_retention,
             legal_hold=legal_hold,
+            cache_control=cache_control,
         )
 
         return copy_file_sim.as_upload_result(account_auth_token)
@@ -1543,8 +1556,8 @@ class RawSimulator(AbstractRawApi):
         large_file_id,
         part_number,
         bytes_range=None,
-        destination_server_side_encryption: Optional[EncryptionSetting] = None,
-        source_server_side_encryption: Optional[EncryptionSetting] = None,
+        destination_server_side_encryption: EncryptionSetting | None = None,
+        source_server_side_encryption: EncryptionSetting | None = None,
     ):
         if destination_server_side_encryption is not None and destination_server_side_encryption.mode == EncryptionMode.SSE_B2:
             raise ValueError(
@@ -1717,10 +1730,11 @@ class RawSimulator(AbstractRawApi):
         file_name,
         content_type,
         file_info,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         bucket = self._get_bucket_by_id(bucket_id)
         self._assert_account_auth(api_url, account_auth_token, bucket.account_id, 'writeFiles')
@@ -1733,6 +1747,7 @@ class RawSimulator(AbstractRawApi):
             file_retention,
             legal_hold,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
         self.file_id_to_bucket_id[result['fileId']] = bucket_id
 
@@ -1747,12 +1762,12 @@ class RawSimulator(AbstractRawApi):
         bucket_type=None,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
         if_revision_is=None,
-        default_server_side_encryption: Optional[EncryptionSetting] = None,
-        default_retention: Optional[BucketRetentionSetting] = None,
-        replication: Optional[ReplicationConfiguration] = None,
-        is_file_lock_enabled: Optional[bool] = None,
+        default_server_side_encryption: EncryptionSetting | None = None,
+        default_retention: BucketRetentionSetting | None = None,
+        replication: ReplicationConfiguration | None = None,
+        is_file_lock_enabled: bool | None = None,
     ):
         assert bucket_type or bucket_info or cors_rules or lifecycle_rules or default_server_side_encryption or replication or is_file_lock_enabled is not None
         bucket = self._get_bucket_by_id(bucket_id)
@@ -1777,11 +1792,12 @@ class RawSimulator(AbstractRawApi):
         content_length: int,
         content_type: str,
         content_sha1: str,
-        file_infos: dict,
-        server_side_encryption: Optional[EncryptionSetting],
-        file_retention: Optional[FileRetentionSetting],
-        legal_hold: Optional[LegalHold],
-        custom_upload_timestamp: Optional[int] = None,
+        file_info: dict,
+        server_side_encryption: EncryptionSetting | None,
+        file_retention: FileRetentionSetting | None,
+        legal_hold: LegalHold | None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ) -> dict:
 
         # fix to allow calculating headers on unknown key - only for simulation
@@ -1796,11 +1812,12 @@ class RawSimulator(AbstractRawApi):
             content_length=content_length,
             content_type=content_type,
             content_sha1=content_sha1,
-            file_infos=file_infos,
+            file_info=file_info,
             server_side_encryption=server_side_encryption,
             file_retention=file_retention,
             legal_hold=legal_hold,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def upload_file(
@@ -1811,12 +1828,13 @@ class RawSimulator(AbstractRawApi):
         content_length: int,
         content_type: str,
         content_sha1: str,
-        file_infos: dict,
+        file_info: dict,
         data_stream,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         with ConcurrentUsedAuthTokenGuard(
             self.currently_used_auth_tokens[upload_auth_token], upload_auth_token
@@ -1833,7 +1851,7 @@ class RawSimulator(AbstractRawApi):
                 assert server_side_encryption.mode in (
                     EncryptionMode.NONE, EncryptionMode.SSE_B2, EncryptionMode.SSE_C
                 )
-                file_infos = server_side_encryption.add_key_id_to_file_info(file_infos)
+                file_info = server_side_encryption.add_key_id_to_file_info(file_info)
 
             # we don't really need headers further on
             # but we still simulate their calculation
@@ -1843,11 +1861,12 @@ class RawSimulator(AbstractRawApi):
                 content_length=content_length,
                 content_type=content_type,
                 content_sha1=content_sha1,
-                file_infos=file_infos,
+                file_info=file_info,
                 server_side_encryption=server_side_encryption,
                 file_retention=file_retention,
                 legal_hold=legal_hold,
                 custom_upload_timestamp=custom_upload_timestamp,
+                cache_control=cache_control,
             )
 
             response = bucket.upload_file(
@@ -1857,12 +1876,13 @@ class RawSimulator(AbstractRawApi):
                 content_length,
                 content_type,
                 content_sha1,
-                file_infos,
+                file_info,
                 data_stream,
                 server_side_encryption,
                 file_retention,
                 legal_hold,
                 custom_upload_timestamp,
+                cache_control,
             )
             file_id = response['fileId']
             self.file_id_to_bucket_id[file_id] = bucket_id
@@ -1877,7 +1897,7 @@ class RawSimulator(AbstractRawApi):
         content_length,
         sha1_sum,
         input_stream,
-        server_side_encryption: Optional[EncryptionSetting] = None,
+        server_side_encryption: EncryptionSetting | None = None,
     ):
         with ConcurrentUsedAuthTokenGuard(
             self.currently_used_auth_tokens[upload_auth_token], upload_auth_token
