@@ -7,23 +7,24 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
 
-from functools import partial
-from enum import Enum, unique
-from typing import Any, Dict, Optional
 import logging
+from enum import Enum, unique
+from functools import partial
+from typing import Any
 
 from b2sdk.account_info.abstract import AbstractAccountInfo
-from b2sdk.account_info.sqlite_account_info import SqliteAccountInfo
 from b2sdk.account_info.exception import MissingAccountData
+from b2sdk.account_info.sqlite_account_info import SqliteAccountInfo
+from b2sdk.api_config import DEFAULT_HTTP_API_CONFIG, B2HttpApiConfig
 from b2sdk.b2http import B2Http
 from b2sdk.cache import AbstractCache, AuthInfoCache, DummyCache
 from b2sdk.encryption.setting import EncryptionSetting
-from b2sdk.replication.setting import ReplicationConfiguration
-from b2sdk.exception import (InvalidAuthToken, Unauthorized)
+from b2sdk.exception import InvalidAuthToken, Unauthorized
 from b2sdk.file_lock import BucketRetentionSetting, FileRetentionSetting, LegalHold
-from b2sdk.raw_api import ALL_CAPABILITIES, REALM_URLS
-from b2sdk.api_config import B2HttpApiConfig, DEFAULT_HTTP_API_CONFIG
+from b2sdk.raw_api import ALL_CAPABILITIES, REALM_URLS, LifecycleRule
+from b2sdk.replication.setting import ReplicationConfiguration
 
 logger = logging.getLogger(__name__)
 
@@ -46,8 +47,8 @@ class B2Session:
 
     def __init__(
         self,
-        account_info: Optional[AbstractAccountInfo] = None,
-        cache: Optional[AbstractCache] = None,
+        account_info: AbstractAccountInfo | None = None,
+        cache: AbstractCache | None = None,
         api_config: B2HttpApiConfig = DEFAULT_HTTP_API_CONFIG
     ):
         """
@@ -144,10 +145,10 @@ class B2Session:
         bucket_type,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
         default_server_side_encryption=None,
-        is_file_lock_enabled: Optional[bool] = None,
-        replication: Optional[ReplicationConfiguration] = None,
+        is_file_lock_enabled: bool | None = None,
+        replication: ReplicationConfiguration | None = None,
     ):
         return self._wrap_default_token(
             self.raw_api.create_bucket,
@@ -184,9 +185,7 @@ class B2Session:
     def delete_file_version(self, file_id, file_name):
         return self._wrap_default_token(self.raw_api.delete_file_version, file_id, file_name)
 
-    def download_file_from_url(
-        self, url, range_=None, encryption: Optional[EncryptionSetting] = None
-    ):
+    def download_file_from_url(self, url, range_=None, encryption: EncryptionSetting | None = None):
         return self._wrap_token(
             self.raw_api.download_file_from_url,
             TokenType.API_TOKEN_ONLY,
@@ -204,10 +203,10 @@ class B2Session:
             valid_duration_in_seconds
         )
 
-    def get_file_info_by_id(self, file_id: str) -> Dict[str, Any]:
+    def get_file_info_by_id(self, file_id: str) -> dict[str, Any]:
         return self._wrap_default_token(self.raw_api.get_file_info_by_id, file_id)
 
-    def get_file_info_by_name(self, bucket_name: str, file_name: str) -> Dict[str, Any]:
+    def get_file_info_by_name(self, bucket_name: str, file_name: str) -> dict[str, Any]:
         return self._wrap_default_token(self.raw_api.get_file_info_by_name, bucket_name, file_name)
 
     def get_upload_url(self, bucket_id):
@@ -293,10 +292,11 @@ class B2Session:
         file_name,
         content_type,
         file_info,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         return self._wrap_default_token(
             self.raw_api.start_large_file,
@@ -308,6 +308,7 @@ class B2Session:
             file_retention=file_retention,
             legal_hold=legal_hold,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def update_bucket(
@@ -317,12 +318,12 @@ class B2Session:
         bucket_type=None,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None,
+        lifecycle_rules: list[LifecycleRule] | None = None,
         if_revision_is=None,
-        default_server_side_encryption: Optional[EncryptionSetting] = None,
-        default_retention: Optional[BucketRetentionSetting] = None,
-        replication: Optional[ReplicationConfiguration] = None,
-        is_file_lock_enabled: Optional[bool] = None,
+        default_server_side_encryption: EncryptionSetting | None = None,
+        default_retention: BucketRetentionSetting | None = None,
+        replication: ReplicationConfiguration | None = None,
+        is_file_lock_enabled: bool | None = None,
     ):
         return self._wrap_default_token(
             self.raw_api.update_bucket,
@@ -346,12 +347,13 @@ class B2Session:
         content_length,
         content_type,
         content_sha1,
-        file_infos,
+        file_info,
         data_stream,
-        server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         return self._wrap_token(
             self.raw_api.upload_file,
@@ -361,12 +363,13 @@ class B2Session:
             content_length,
             content_type,
             content_sha1,
-            file_infos,
+            file_info,
             data_stream,
             server_side_encryption,
             file_retention=file_retention,
             legal_hold=legal_hold,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
 
     def upload_part(
@@ -376,7 +379,7 @@ class B2Session:
         content_length,
         sha1_sum,
         input_stream,
-        server_side_encryption: Optional[EncryptionSetting] = None,
+        server_side_encryption: EncryptionSetting | None = None,
     ):
         return self._wrap_token(
             self.raw_api.upload_part,
@@ -406,10 +409,11 @@ class B2Session:
         content_type=None,
         file_info=None,
         destination_bucket_id=None,
-        destination_server_side_encryption: Optional[EncryptionSetting] = None,
-        source_server_side_encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        destination_server_side_encryption: EncryptionSetting | None = None,
+        source_server_side_encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        cache_control: str | None = None,
     ):
         return self._wrap_default_token(
             self.raw_api.copy_file,
@@ -424,6 +428,7 @@ class B2Session:
             source_server_side_encryption=source_server_side_encryption,
             file_retention=file_retention,
             legal_hold=legal_hold,
+            cache_control=cache_control,
         )
 
     def copy_part(
@@ -432,8 +437,8 @@ class B2Session:
         large_file_id,
         part_number,
         bytes_range=None,
-        destination_server_side_encryption: Optional[EncryptionSetting] = None,
-        source_server_side_encryption: Optional[EncryptionSetting] = None,
+        destination_server_side_encryption: EncryptionSetting | None = None,
+        source_server_side_encryption: EncryptionSetting | None = None,
     ):
         return self._wrap_default_token(
             self.raw_api.copy_part,
