@@ -7,16 +7,16 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
 
 import logging
 import threading
-
 from abc import ABCMeta, abstractmethod
-from typing import Dict, Optional
+from typing import TYPE_CHECKING
 
 from b2sdk.encryption.setting import EncryptionSetting
 from b2sdk.exception import MaxFileSizeExceeded
-from b2sdk.file_lock import FileRetentionSetting, LegalHold, NO_RETENTION_FILE_SETTING
+from b2sdk.file_lock import NO_RETENTION_FILE_SETTING, FileRetentionSetting, LegalHold
 from b2sdk.http_constants import LARGE_FILE_SHA1
 from b2sdk.transfer.outbound.large_file_upload_state import LargeFileUploadState
 from b2sdk.transfer.outbound.upload_source import UploadSourceStream
@@ -25,6 +25,10 @@ AUTO_CONTENT_TYPE = 'b2/x-auto'
 
 logger = logging.getLogger(__name__)
 
+if TYPE_CHECKING:
+    from b2sdk.transfer.emerge.planner.part_definition import UploadEmergePartDefinition
+    from b2sdk.transfer.emerge.planner.planner import StreamingEmergePlan
+
 
 class EmergeExecutor:
     def __init__(self, services):
@@ -40,10 +44,11 @@ class EmergeExecutor:
         progress_listener,
         continue_large_file_id=None,
         max_queue_size=None,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         if emerge_plan.is_large_file():
             execution = LargeFileEmergeExecution(
@@ -59,6 +64,7 @@ class EmergeExecutor:
                 continue_large_file_id=continue_large_file_id,
                 max_queue_size=max_queue_size,
                 custom_upload_timestamp=custom_upload_timestamp,
+                cache_control=cache_control,
             )
         else:
             if continue_large_file_id is not None:
@@ -74,6 +80,7 @@ class EmergeExecutor:
                 file_retention=file_retention,
                 legal_hold=legal_hold,
                 custom_upload_timestamp=custom_upload_timestamp,
+                cache_control=cache_control,
             )
         return execution.execute_plan(emerge_plan)
 
@@ -89,10 +96,11 @@ class BaseEmergeExecution(metaclass=ABCMeta):
         content_type,
         file_info,
         progress_listener,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         self.services = services
         self.bucket_id = bucket_id
@@ -104,6 +112,7 @@ class BaseEmergeExecution(metaclass=ABCMeta):
         self.file_retention = file_retention
         self.legal_hold = legal_hold
         self.custom_upload_timestamp = custom_upload_timestamp
+        self.cache_control = cache_control
 
     @abstractmethod
     def execute_plan(self, emerge_plan):
@@ -132,14 +141,15 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
         content_type,
         file_info,
         progress_listener,
-        encryption: Optional[EncryptionSetting] = None,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        encryption: EncryptionSetting | None = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
         continue_large_file_id=None,
         max_queue_size=None,
-        custom_upload_timestamp: Optional[int] = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
-        super(LargeFileEmergeExecution, self).__init__(
+        super().__init__(
             services,
             bucket_id,
             file_name,
@@ -150,6 +160,7 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
             file_retention=file_retention,
             legal_hold=legal_hold,
             custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
         )
         self.continue_large_file_id = continue_large_file_id
         self.max_queue_size = max_queue_size
@@ -157,7 +168,7 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
         if self.max_queue_size is not None:
             self._semaphore = threading.Semaphore(self.max_queue_size)
 
-    def execute_plan(self, emerge_plan):
+    def execute_plan(self, emerge_plan: StreamingEmergePlan):
         total_length = emerge_plan.get_total_length()
         encryption = self.encryption
 
@@ -186,6 +197,7 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
             legal_hold=self.legal_hold,
             emerge_parts_dict=emerge_parts_dict,
             custom_upload_timestamp=self.custom_upload_timestamp,
+            cache_control=self.cache_control,
         )
 
         if unfinished_file is None:
@@ -201,6 +213,7 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
                 encryption=encryption,
                 file_retention=self.file_retention,
                 legal_hold=self.legal_hold,
+                cache_control=self.cache_control,
             )
         file_id = unfinished_file.file_id
 
@@ -231,7 +244,7 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
         response = self.services.session.finish_large_file(file_id, part_sha1_array)
         return self.services.api.file_version_factory.from_api_response(response)
 
-    def _execute_step(self, execution_step):
+    def _execute_step(self, execution_step: UploadPartExecutionStep):
         semaphore = self._semaphore
         if semaphore is None:
             return execution_step.execute()
@@ -253,10 +266,11 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
         file_info,
         continue_large_file_id,
         encryption: EncryptionSetting,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
         emerge_parts_dict=None,
-        custom_upload_timestamp: Optional[int] = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         if 'listFiles' not in self.services.session.account_info.get_allowed()['capabilities']:
             return None, {}
@@ -289,6 +303,7 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
                 file_retention,
                 legal_hold,
                 custom_upload_timestamp=custom_upload_timestamp,
+                cache_control=cache_control,
             )
         elif emerge_parts_dict is not None:
             unfinished_file, finished_parts = self._match_unfinished_file_if_possible(
@@ -300,9 +315,147 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
                 file_retention,
                 legal_hold,
                 custom_upload_timestamp=custom_upload_timestamp,
+                cache_control=cache_control,
             )
         return unfinished_file, finished_parts
 
+    def _find_matching_unfinished_file(
+        self,
+        bucket_id,
+        file_name,
+        file_info,
+        emerge_parts_dict,
+        encryption: EncryptionSetting,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
+        check_file_info_without_large_file_sha1: bool | None = False,
+        eager_mode: bool | None = False,
+    ):
+        """
+        Search for a matching unfinished large file in the specified bucket.
+
+        In case a matching file is found but has inconsistencies (for example, mismatching file info or encryption settings),
+        mismatches are logged.
+
+        :param bucket_id: The identifier of the bucket where the unfinished file resides.
+        :param file_name: The name of the file to be matched.
+        :param file_info: Information about the file to be uploaded.
+        :param emerge_parts_dict: A dictionary containing the parts of the file to be emerged.
+        :param encryption: The encryption settings for the file.
+        :param file_retention: The retention settings for the file, if any.
+        :param legal_hold: The legal hold status of the file, if any.
+        :param custom_upload_timestamp: The custom timestamp for the upload, if any.
+        :param cache_control: The cache control settings for the file, if any.
+        :param check_file_info_without_large_file_sha1: A flag indicating whether the file information should be checked without the `large_file_sha1`.
+        :param eager_mode: A flag indicating whether the first matching file should be returned.
+        
+        :return: A tuple of the best matching unfinished file and its finished parts. If no match is found, returns `None`.
+        """
+
+        file_retention = file_retention or NO_RETENTION_FILE_SETTING
+        best_match_file = None
+        best_match_parts = {}
+        best_match_parts_len = 0
+
+        for file_ in self.services.large_file.list_unfinished_large_files(
+            bucket_id, prefix=file_name
+        ):
+            if file_.file_name != file_name:
+                logger.debug('Rejecting %s: file name mismatch', file_.file_id)
+                continue
+
+            if file_.file_info != file_info:
+                if check_file_info_without_large_file_sha1:
+                    file_info_without_large_file_sha1 = self._get_file_info_without_large_file_sha1(
+                        file_info
+                    )
+                    if file_info_without_large_file_sha1 != self._get_file_info_without_large_file_sha1(
+                        file_.file_info
+                    ):
+                        logger.debug(
+                            'Rejecting %s: file info mismatch after dropping `large_file_sha1`',
+                            file_.file_id
+                        )
+                        continue
+                else:
+                    logger.debug('Rejecting %s: file info mismatch', file_.file_id)
+                    continue
+
+            if encryption is not None and encryption != file_.encryption:
+                logger.debug('Rejecting %s: encryption mismatch', file_.file_id)
+                continue
+
+            if cache_control is not None and cache_control != file_.cache_control:
+                logger.debug('Rejecting %s: cacheControl mismatch', file_.file_id)
+                continue
+
+            if legal_hold is None:
+                if LegalHold.UNSET != file_.legal_hold:
+                    logger.debug('Rejecting %s: legal hold mismatch (not unset)', file_.file_id)
+                    continue
+            elif legal_hold != file_.legal_hold:
+                logger.debug('Rejecting %s: legal hold mismatch', file_.file_id)
+                continue
+
+            if file_retention != file_.file_retention:
+                logger.debug('Rejecting %s: retention mismatch', file_.file_id)
+                continue
+
+            if custom_upload_timestamp is not None and file_.upload_timestamp != custom_upload_timestamp:
+                logger.debug('Rejecting %s: custom_upload_timestamp mismatch', file_.file_id)
+                continue
+
+            finished_parts = {}
+
+            for part in self.services.large_file.list_parts(file_.file_id):
+
+                emerge_part = emerge_parts_dict.get(part.part_number)
+
+                if emerge_part is None:
+                    # something is wrong - we have a part that we don't know about
+                    # so we can't resume this upload
+                    logger.debug(
+                        'Rejecting %s: part %s not found in emerge parts, giving up.',
+                        file_.file_id, part.part_number
+                    )
+                    finished_parts = None
+                    break
+
+                # Compare part sizes
+                if emerge_part.get_length() != part.content_length:
+                    logger.debug(
+                        'Rejecting %s: part %s size mismatch', file_.file_id, part.part_number
+                    )
+                    continue  # part size doesn't match - so we reupload
+
+                # Compare part hashes
+                if emerge_part.is_hashable() and emerge_part.get_sha1() != part.content_sha1:
+                    logger.debug(
+                        'Rejecting %s: part %s sha1 mismatch', file_.file_id, part.part_number
+                    )
+                    continue  # part.sha1 doesn't match - so we reupload
+
+                finished_parts[part.part_number] = part
+
+            if finished_parts is None:
+                continue
+
+            finished_parts_len = len(finished_parts)
+
+            if finished_parts and (
+                best_match_file is None or finished_parts_len > best_match_parts_len
+            ):
+                best_match_file = file_
+                best_match_parts = finished_parts
+                best_match_parts_len = finished_parts_len
+
+            if eager_mode and best_match_file is not None:
+                break
+
+        return best_match_file, best_match_parts
+
     def _find_unfinished_file_by_plan_id(
         self,
         bucket_id,
@@ -310,66 +463,54 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
         file_info,
         emerge_parts_dict,
         encryption: EncryptionSetting,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
-        file_retention = file_retention or NO_RETENTION_FILE_SETTING
-        assert 'plan_id' in file_info
-        best_match_file = None
-        best_match_parts = {}
-        best_match_parts_len = 0
-        for file_ in self.services.large_file.list_unfinished_large_files(
-            bucket_id, prefix=file_name
-        ):
-            if file_.file_info != file_info:
-                continue
-            # FIXME: encryption is None ???
-            if encryption is None or file_.encryption != encryption:
-                continue
+        """
+        Search for a matching unfinished large file by plan_id in the specified bucket.
 
-            if legal_hold is None:
-                if LegalHold.UNSET != file_.legal_hold:
-                    # Uploading and not providing legal_hold means that server's response about that file version
-                    # will have legal_hold=LegalHold.UNSET
-                    continue
-            elif legal_hold != file_.legal_hold:
-                continue
+        This function aims to locate a matching unfinished large file using the plan_id and the supplied parameters. 
+        It's used to resume an interrupted upload, centralizing the shared logic between `_find_unfinished_file_by_plan_id` 
+        and `_match_unfinished_file_if_possible`.
 
-            if file_retention != file_.file_retention:
-                # if `file_.file_retention` is UNKNOWN then we skip - lib user can still
-                # pass UNKNOWN file_retention here - but raw_api/server won't allow it
-                # and we don't check it here
-                continue
+        In case a matching file is found but has inconsistencies (for example, mismatching file info or encryption settings), 
+        the function checks if 'plan_id' is in file_info, as this is a prerequisite.
 
-            if custom_upload_timestamp is not None and file_.upload_timestamp != custom_upload_timestamp:
-                continue
+        :param bucket_id: The identifier of the bucket where the unfinished file resides.
+        :param file_name: The name of the file to be matched.
+        :param file_info: Information about the file to be uploaded.
+        :param emerge_parts_dict: A dictionary containing the parts of the file to be emerged.
+        :param encryption: The encryption settings for the file.
+        :param file_retention: The retention settings for the file, if any.
+        :param legal_hold: The legal hold status of the file, if any.
+        :param custom_upload_timestamp: The custom timestamp for the upload, if any.
+        :param cache_control: The cache control settings for the file, if any.
+        
+        :return: A tuple of the best matching unfinished file and its finished parts. If no match is found, it returns `None`.
+        """
+        if 'plan_id' not in file_info:
+            raise ValueError("The 'plan_id' key must be in file_info dictionary.")
 
-            finished_parts = {}
-            for part in self.services.large_file.list_parts(file_.file_id):
-                emerge_part = emerge_parts_dict.get(part.part_number)
-                if emerge_part is None:
-                    # large file with same `plan_id` has more parts than current plan
-                    # so we want to skip this large file because it is broken
-                    finished_parts = None
-                    break
-                if emerge_part.is_hashable() and emerge_part.get_sha1() != part.content_sha1:
-                    continue  # auto-healing - `plan_id` matches but part.sha1 doesn't - so we reupload
-                finished_parts[part.part_number] = part
-            if finished_parts is None:
-                continue
-            finished_parts_len = len(finished_parts)
-            if best_match_file is None or finished_parts_len > best_match_parts_len:
-                best_match_file = file_
-                best_match_parts = finished_parts
-                best_match_parts_len = finished_parts_len
-        return best_match_file, best_match_parts
+        return self._find_matching_unfinished_file(
+            bucket_id=bucket_id,
+            file_name=file_name,
+            file_info=file_info,
+            emerge_parts_dict=emerge_parts_dict,
+            encryption=encryption,
+            file_retention=file_retention or NO_RETENTION_FILE_SETTING,
+            legal_hold=legal_hold,
+            custom_upload_timestamp=custom_upload_timestamp,
+            cache_control=cache_control,
+            check_file_info_without_large_file_sha1=False,
+        )
 
     @classmethod
     def _get_file_info_without_large_file_sha1(
         cls,
-        file_info: Optional[Dict[str, str]],
-    ) -> Optional[Dict[str, str]]:
+        file_info: dict[str, str] | None,
+    ) -> dict[str, str] | None:
         if not file_info or LARGE_FILE_SHA1 not in file_info:
             return file_info
         out_file_info = dict(file_info)
@@ -383,108 +524,61 @@ class LargeFileEmergeExecution(BaseEmergeExecution):
         file_info,
         emerge_parts_dict,
         encryption: EncryptionSetting,
-        file_retention: Optional[FileRetentionSetting] = None,
-        legal_hold: Optional[LegalHold] = None,
-        custom_upload_timestamp: Optional[int] = None,
+        file_retention: FileRetentionSetting | None = None,
+        legal_hold: LegalHold | None = None,
+        custom_upload_timestamp: int | None = None,
+        cache_control: str | None = None,
     ):
         """
-        Find an unfinished file that may be used to resume a large file upload.  The
-        file is found using the filename and comparing the uploaded parts against
-        the local file.
+        Scan for a suitable unfinished large file in the specified bucket to resume upload.
 
-        This is only possible if the application key being used allows ``listFiles`` access.
+        This function examines each unfinished large file for a possible match with the provided
+        parameters. This enables resumption of an interrupted upload by reusing the unfinished file,
+        provided that file's info and additional parameters match.
+
+        Along with the filename and file info, additional parameters like encryption, file retention,
+        legal hold, custom upload timestamp, and cache control are compared for a match. The
+        'emerge_parts_dict' is also cross-checked for matching file parts.
+
+        Function is eager to find a match, and will return the first match it finds. If no match is
+        found, it returns `None`.
+
+        :param bucket_id: The identifier of the bucket containing the unfinished file.
+        :param file_name: The name of the file to find.
+        :param file_info: Information about the file to be uploaded.
+        :param emerge_parts_dict: A dictionary of the parts of the file to be emerged.
+        :param encryption: The encryption settings for the file.
+        :param file_retention: The retention settings for the file, if applicable.
+        :param legal_hold: The legal hold status of the file, if applicable.
+        :param custom_upload_timestamp: The custom timestamp for the upload, if set.
+        :param cache_control: The cache control settings for the file, if set.
+        
+        :return: A tuple of the best matching unfinished file and its finished parts. If no match is found, returns `None`.
         """
-        file_retention = file_retention or NO_RETENTION_FILE_SETTING
-        file_info_without_large_file_sha1 = self._get_file_info_without_large_file_sha1(file_info)
-
         logger.debug('Checking for matching unfinished large files for %s...', file_name)
-        for file_ in self.services.large_file.list_unfinished_large_files(
-            bucket_id, prefix=file_name
-        ):
-            if file_.file_name != file_name:
-                logger.debug('Rejecting %s: file has a different file name', file_.file_id)
-                continue
-            if file_.file_info != file_info:
-                if (LARGE_FILE_SHA1 in file_.file_info) == (LARGE_FILE_SHA1 in file_info):
-                    logger.debug(
-                        'Rejecting %s: large_file_sha1 is present or missing in both file infos',
-                        file_.file_id
-                    )
-                    continue
 
-                if self._get_file_info_without_large_file_sha1(
-                    file_.file_info
-                ) != file_info_without_large_file_sha1:
-                    # ignoring the large_file_sha1 file infos are still different
-                    logger.debug(
-                        'Rejecting %s: file info mismatch after dropping `large_file_sha1`',
-                        file_.file_id
-                    )
-                    continue
+        file_, finished_parts = self._find_matching_unfinished_file(
+            bucket_id,
+            file_name,
+            file_info,
+            emerge_parts_dict,
+            encryption,
+            file_retention,
+            legal_hold,
+            custom_upload_timestamp,
+            cache_control,
+            check_file_info_without_large_file_sha1=True,
+            eager_mode=True,
+        )
 
-            # FIXME: what if `encryption is None` - match ANY encryption? :)
-            if encryption is not None and encryption != file_.encryption:
-                logger.debug('Rejecting %s: encryption mismatch', file_.file_id)
-                continue
+        if file_ is None:
+            logger.debug('No matching unfinished files found.')
+            return None, {}
 
-            if legal_hold is None:
-                if LegalHold.UNSET != file_.legal_hold:
-                    # Uploading and not providing legal_hold means that server's response about that file version
-                    # will have legal_hold=LegalHold.UNSET
-                    logger.debug('Rejecting %s: legal hold mismatch (not unset)', file_.file_id)
-                    continue
-            elif legal_hold != file_.legal_hold:
-                logger.debug('Rejecting %s: legal hold mismatch', file_.file_id)
-                continue
-
-            if file_retention != file_.file_retention:
-                # if `file_.file_retention` is UNKNOWN then we skip - lib user can still
-                # pass UNKNOWN file_retention here - but raw_api/server won't allow it
-                # and we don't check it here
-                logger.debug('Rejecting %s: retention mismatch', file_.file_id)
-                continue
-
-            if custom_upload_timestamp is not None and file_.upload_timestamp != custom_upload_timestamp:
-                logger.debug('Rejecting %s: custom_upload_timestamp mismatch', file_.file_id)
-                continue
-
-            files_match = True
-            finished_parts = {}
-            for part in self.services.large_file.list_parts(file_.file_id):
-                emerge_part = emerge_parts_dict.get(part.part_number)
-                if emerge_part is None:
-                    files_match = False
-                    break
-
-                # Compare part sizes
-                if emerge_part.get_length() != part.content_length:
-                    files_match = False
-                    break
-
-                # Compare hash
-                assert emerge_part.is_hashable()
-                sha1_sum = emerge_part.get_sha1()
-                if sha1_sum != part.content_sha1:
-                    files_match = False
-                    break
-
-                # Save part
-                finished_parts[part.part_number] = part
-
-            # Skip not matching files or unfinished files with no uploaded parts
-            if not files_match or not finished_parts:
-                logger.debug('Rejecting %s: No finished parts or part mismatch', file_.file_id)
-                continue
-
-            # Return first matched file
-            logger.debug(
-                'Unfinished file %s matches with %i finished parts', file_.file_id,
-                len(finished_parts)
-            )
-            return file_, finished_parts
-
-        logger.debug('No matching unfinished files found.')
-        return None, {}
+        logger.debug(
+            'Unfinished file %s matches with %i finished parts', file_.file_id, len(finished_parts)
+        )
+        return file_, finished_parts
 
 
 class BaseExecutionStepFactory(metaclass=ABCMeta):
@@ -521,13 +615,13 @@ class LargeFileEmergeExecutionStepFactory(BaseExecutionStepFactory):
     def __init__(
         self,
         emerge_execution,
-        emerge_part,
+        emerge_part: UploadEmergePartDefinition,
         part_number,
         large_file_id,
         large_file_upload_state,
         finished_parts=None,
     ):
-        super(LargeFileEmergeExecutionStepFactory, self).__init__(emerge_execution, emerge_part)
+        super().__init__(emerge_execution, emerge_part)
         self.part_number = part_number
         self.large_file_id = large_file_id
         self.large_file_upload_state = large_file_upload_state
@@ -589,6 +683,7 @@ class CopyFileExecutionStep(BaseExecutionStep):
             source_encryption=self.copy_source_range.encryption,
             file_retention=execution.file_retention,
             legal_hold=execution.legal_hold,
+            cache_control=execution.cache_control,
         )
 
 
@@ -646,6 +741,7 @@ class UploadFileExecutionStep(BaseExecutionStep):
             file_retention=execution.file_retention,
             legal_hold=execution.legal_hold,
             custom_upload_timestamp=execution.custom_upload_timestamp,
+            cache_control=execution.cache_control,
         )
 
 
