@@ -7,6 +7,7 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
 
 from b2sdk._v3 import *  # noqa
 from b2sdk._v3 import parse_folder as parse_sync_folder
@@ -20,3 +21,17 @@ from .bucket import Bucket, BucketFactory
 from .session import B2Session
 from .sync import B2SyncPath
 from .transfer import DownloadManager, UploadManager
+
+# version & version utils
+
+from .version_utils import rename_argument, rename_function
+
+# raw_simulator
+
+from .raw_simulator import BucketSimulator
+from .raw_simulator import RawSimulator
+
+# raw_api
+
+from .raw_api import AbstractRawApi
+from .raw_api import B2RawHTTPApi
